"""
DBGET database
"""
from __future__ import absolute_import

import re

from . import entry
from .entry import fields
from . import api

def iter_take(source_iter, n):
    source_iter = iter(source_iter)
    return [item for _, item in zip(range(n), source_iter)]

def batch_iter(source_iter, n):
    source_iter = iter(source_iter)
    while True:
        batch = iter_take(source_iter, n)
        if batch:
            yield batch
        else:
            break
        
def chain_iter(chains_iter):
    for iter in chains_iter:
        for element in iter:
            yield element

class DBDataBase(object):
    """ A wrapper for DBGET database.
    """
    # ENTRY_TYPE constructor (type)
    ENTRY_TYPE = entry.DBEntry
    
    # Needs to be set in a subclass or object instance 
    DB = None
    
    def __init__(self, **kwargs):
        if not self.DB:
            raise TypeError("Cannot make an instance of abstract base class %r." \
                            % type(self).__name__)
            
        self.api = api.CachedKeggApi()
        self.info = self.api.binfo(self.DB)
        release = self.info.release
        self.api.set_default_release(release)
        self._keys = []
        
    def keys(self):
        return list(self._keys)
    
    def iterkeys(self):
        return iter(self._keys)
    
    def items(self):
        return list(zip(self.keys(), self.batch_get(self.keys())))
    
    def iteritems(self):
        batch_size = 100
        iterkeys = self.iterkeys()
        return chain_iter(zip(batch, self.batch_get(batch))
                          for batch in batch_iter(iterkeys, batch_size))
        
#        return ((key, self.__getitem__(key)) for key in self.iterkeys())
    
    def values(self):
        return self.batch_get(self.keys())
    
    def itervalues(self):
        batch_size = 100
        iterkeys = self.iterkeys()
        return chain_iter(self.batch_get(batch)
                          for batch in batch_iter(iterkeys, batch_size))
        
#        return (self.__getitem__(key) for key in self.iterkeys())
    
    def get(self, key, default=None):
        try:
            return self.__getitem__(key)
        except KeyError:
            return default
        
    def has_key(self, key):
        return self.__contains__(key)
    
    def __getitem__(self, key):
        e = self.get_entry(key)
        if e is None:
            raise KeyError(key)
        else:
            return e
    
    def __contains__(self, key):
        return key in set(self.keys())
    
    def __len__(self):
        return len(self.keys())
    
    def __iter__(self):
        return iter(self.keys())
    
    def get_text(self, key):
        key = self._add_db(key)
        return self.api.bget([key])
    
    def get_entry(self, key):
        text = self.get_text(key)
        if not text or text == "None":
            return None
        else:
            return self.ENTRY_TYPE(text)
        
    def find(self, name):
        """ Find ``name`` using BFIND. 
        """
        res = self.api.bfind(self.DB, name).splitlines()
        return [r.split(" ", 1)[0] for r in res]    
        
    def pre_cache(self, keys=None, batch_size=100, progress_callback=None):
        """ Retrive all the entries and cache them locally.
        """
        # TODO do this in multiple threads
    
        if not isinstance(self.api, api.CachedKeggApi):
            raise TypeError("Not an an instance of api.CachedKeggApi")
        
        if batch_size > 100 or batch_size < 1:
            raise ValueError("Invalid batch_size")
        
        if keys is None:
            keys = self.keys()
            
        keys = list(keys)
        start = 0
        while start < len(keys):
            batch = keys[start: start + batch_size]
            batch = map(self._add_db, batch)
            
            self.api.bget(batch)
            
            if progress_callback:
                progress_callback(100.0 * start / len(keys))
                
            start += batch_size
            
    def batch_get(self, keys):
        """ Batch retrieve all entries for keys. This can be
        significantly faster then getting each entry separately
        especially if entries are not yet cached.
        
        """
        entries = []
        batch_size = 100
        keys = list(keys)
        start = 0
        while start < len(keys):
            batch = keys[start: start + batch_size]
            batch = map(self._add_db, batch)
            batch_entries = self.api.bget(batch)
            if batch_entries is not None:
                batch_entries = batch_entries.split("///\n")
                # Remove possible empty last line  
                batch_entries = [e for e in batch_entries if e.strip()]
                entries.extend(map(self.ENTRY_TYPE, batch_entries))
            start += batch_size
            
        return entries
            
    def _add_db(self, key):
        """ Prefix the key with '%(DB)s:' string if not already
        prefixed. 
        """
        if not key.startswith(self.DB + ":"):
            return self.DB + ":" + key
        else:
            return key
        
    @property
    def entries(self):
        return self.values()
    
@entry.entry_decorate
class GenomeEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("DEFINITION", fields.DBDefinitionField),
              ("ANNOTATION", fields.DBSimpleField),
              ("TAXONOMY", fields.DBTaxonomyField),
              ("DATA_SOURCE", fields.DBSimpleField),
              ("ORIGINAL_DB", fields.DBSimpleField),
              ("KEYWORDS", fields.DBSimpleField),
              ("DISEASE", fields.DBSimpleField),
              ("COMMENT", fields.DBSimpleField),
              ("CHROMOSOME", fields.DBFieldWithSubsections),
              ("STATISTICS", fields.DBSimpleField),
              ("REFERENCE", fields.DBReference)]
    
    MULTIPLE_FIELDS = ["REFERENCE"]
    
    def __init__(self, text):
        entry.DBEntry.__init__(self, text)
        
    @property
    def entry_key(self):
        """ Primary entry key used for querying.
        
        .. note:: Unlike most of the other entry types this is the
            first listed 'NAME'.
            
        """
        
        return self.name.split(",", 1)[0]

    @property
    def taxid(self):
        return self.TAXONOMY.taxid
            
    def org_code(self):
        if self.name is not None:
            return self.name.split(",")[0]
        else:
            return self.entry.split(" ")[0]
        

class Genome(DBDataBase):
    DB = "genome"
    ENTRY_TYPE = GenomeEntry
    
    # For obiTaxonomy.common_taxids mapping
    TAXID_MAP = {"562": "511145",   # Escherichia coli K-12 MG1655
                 "2104": "272634",  # Mycoplasma pneumoniae M129 
                 "4530": "39947",   # Oryza sativa ssp. japonica cultivar Nipponbare (Japanese rice)
                 "4932" : "559292", # Saccharomyces cerevisiae S288C
                 "4896": "284812",  # Schizosaccharomyces pombe 972h-
                 }
    
    def __init__(self):
        DBDataBase.__init__(self)
        self._keys = [org.entry_id for org in self.api.list_organisms()]
    
    def _key_to_gn_entry_id(self, key):
        res = self.find(key)
        if len(res) == 0:
            raise KeyError("Unknown key")
        elif len(res) > 1:
            raise ValueError("Not a unique key")
        else:
            return res[0]
    
    @classmethod
    def common_organisms(cls):
        return ['ath', 'bta', 'cel', 'cre', 'dre', 'ddi',
                'dme', 'eco', 'hsa', 'mmu', 'mpn', 'osa',
                'pfa', 'rno', 'sce', 'spo', 'zma', 'xla']
        
    @classmethod
    def essential_organisms(cls):
        return ['ddi', 'dme', 'hsa', 'mmu', 'sce']
    
    def search(self, string, relevance=False):
        """ Search the genome database for string using ``bfind``.
        """
        if relevance:
            raise NotImplementedError("relevance is no longer supported")
        if string in self.TAXID_MAP:
            string = self.TAXID_MAP[string]
            
        res = self.api.bfind(self.DB, string)
        if not res:
            return []
        
        res = res.splitlines()
        res = [r.split(",", 1)[0] for r in res]
        res = [r.split(" ", 1)[1] for r in res]
        return res
    
    
@entry.entry_decorate
class GeneEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("DEFINITION", fields.DBDefinitionField),
              ("ORGANISM", fields.DBSimpleField),
              ("ORTHOLOGY", fields.DBSimpleField),
              ("DRUG_TARGET", fields.DBSimpleField),
              ("PATHWAY", fields.DBPathway),
              ("MODULE", fields.DBSimpleField),
              ("DISEASE", fields.DBSimpleField),
              ("CLASS", fields.DBSimpleField),
              ("POSITION", fields.DBSimpleField),
              ("MOTIF", fields.DBSimpleField),
              ("DBLINKS", fields.DBDBLinks),
              ("STRUCTURE", fields.DBSimpleField),
              ("AASEQ", fields.DBAASeq),
              ("NTSEQ", fields.DBNTSeq)]
    
    def aliases(self):
        return [self.entry_key] + (self.name.split(",") if self.name else []) + [link[1][0] for link in self.dblinks.items() if self.dblinks]

    @property
    def alt_names(self):
        """ For backwards compatibility.
        """
        return self.aliases()
  
class Genes(DBDataBase):
    DB = None # Needs to be set in __init__ 
    ENTRY_TYPE = GeneEntry
    
    def __init__(self, org_code):
        self.DB = org_code
        self.org_code = org_code
        DBDataBase.__init__(self)
        self._keys = self.api.get_genes_by_organism(org_code)
        
    def gene_aliases(self):
        aliases = {}
        for entry in self.itervalues():
            aliases.update(dict.fromkeys(entry.aliases(), self.org_code + ":" + entry.entry_key()))
        return aliases
    

@entry.entry_decorate
class CompoundEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("FORMULA", fields.DBSimpleField),
              ("MASS", fields.DBSimpleField),
              ("REMARK", fields.DBSimpleField),
              ("REACTION", fields.DBSimpleField),
              ("PATHWAY", fields.DBPathway),
              ("ENZYME", fields.DBSimpleField),
              ("DBLINKS", fields.DBDBLinks),
              ("ATOM", fields.DBSimpleField),
              ("BOND", fields.DBSimpleField)
              ]
    
    
class Compounds(DBDataBase):
    DB = "cpd"
    ENTRY_TYPE = CompoundEntry
    
    def __init__(self):
        DBDataBase.__init__(self)
        self._keys = [] # All keys are not available


@entry.entry_decorate    
class ReactionEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("DEFINITION", fields.DBDefinitionField),
              ("EQUATION", fields.DBSimpleField),
              ("ENZYME", fields.DBSimpleField)
              ]
    
class Reactions(DBDataBase):
    DB = "rn"
    ENTRY_TYPE = ReactionEntry
    
    def __init__(self):
        DBDataBase.__init__(self)
        self._keys = [] # All keys are not available
         
class Brite(DBDataBase):
    DB = "br"
    
class Disease(DBDataBase):
    DB = "ds"
        
class Drug(DBDataBase):
    DB = "dr"
    
@entry.entry_decorate
class EnzymeEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("CLASS", fields.DBSimpleField),
              ("SYSNAME", fields.DBSimpleField),
              ("REACTION", fields.DBSimpleField),
              ("ALL_REAC", fields.DBSimpleField),
              ("SUBSTRATE", fields.DBSimpleField),
              ("PRODUCT", fields.DBSimpleField),
              ("COMMENT", fields.DBSimpleField),
              ("REFERENCE", fields.DBReference),
              ("PATHWAY", fields.DBPathway),
              ("ORTHOLOGY", fields.DBSimpleField),
              ("GENES", fields.DBSimpleField),
              ("DBLINKS", fields.DBDBLinks)
              ]
    
    MULTIPLE_FIELDS = ["REFERENCE"]
    
class Enzymes(DBDataBase):
    DB = "ec"
    ENTRY_TYPE = EnzymeEntry
    
    
@entry.entry_decorate
class OrthologyEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("CLASS", fields.DBSimpleField),
              ("DBLINKS", fields.DBDBLinks),
              ("GENES", fields.DBSimpleField),
              ]
    
class Orthology(DBDataBase):
    DB = "ko"
    ENTRY_TYPE = OrthologyEntry
    
    
@entry.entry_decorate
class PathwayEntry(entry.DBEntry):
    FIELDS = [("ENTRY", fields.DBEntryField),
              ("NAME", fields.DBNameField),
              ("DESCRIPTION", fields.DBSimpleField),
              ("CLASS", fields.DBSimpleField),
              ("PATHWAY_MAP", fields.DBPathwayMapField),
              ("DISEASE", fields.DBSimpleField),
              ("DRUG", fields.DBSimpleField),
              ("DBLINKS", fields.DBDBLinks),
              ("ORGANISM", fields.DBSimpleField),
              ("GENE", fields.DBGeneField),
              ("ENZYME", fields.DBEnzymeField),
              ("COMPOUND", fields.DBCompoundField),
              ("REFERENCE", fields.DBReference),
              ("REL_PATHWAY", fields.DBSimpleField),
              ("KO_PATHWAY", fields.DBSimpleField),
              ]
    
    MULTIPLE_FIELDS = ["REFERENCE"]
    
    @property
    def gene(self):
        if hasattr(self, "GENE"):
            genes = self.GENE._convert()
        else:
            return None
        
        org = self.organism
        org_prefix = ""
        if org:
            match = re.findall(r"\[GN:([a-z]+)\]", org)
            if match:
                org_prefix = match[0] + ":"
        genes = [org_prefix + g for g in genes]
        return genes 
    
class Pathways(DBDataBase):
    DB = "path"
    ENTRY_TYPE = PathwayEntry
    
    def __init__(self):
        DBDataBase.__init__(self)
    


"""
<name>PIPA</name>
<description>Access data from PIPA RNA-Seq database.</description>
<icon>icons/PIPA.png</icon>
<priority>30</priority>
"""

from __future__ import absolute_import

import sys, os
from collections import defaultdict
import math

from Orange.orng import orngEnviron
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import obiDicty

#def pyqtConfigure(object, **kwargs):
#    if hasattr(object, "pyqtConfigure"):
#        object.pyqtConfigure(**kwargs)
#    else:
#        for name, value in kwargs.items():
#            if object.metaObject().indexOfProperty(name) != -1:
#                object.setProperty(name, value)
#            elif object.metaObject().indexOfSignal(name) != -1:
#                object.connect(object.

def tfloat(s):
    try:
        return float(s)
    except:
        return None

class MyTreeWidgetItem(QTreeWidgetItem):

    def __init__(self, parent, *args):
        QTreeWidgetItem.__init__(self, parent, *args)
        self.par = parent

    def __contains__(self, text):
        return any(text.upper() in str(self.text(i)).upper() for i in range(self.columnCount()))    
 
    def __lt__(self, o1):
        col = self.par.sortColumn()
        if col in [8,9,10]: #WARNING: hardcoded column numbers
            return tfloat(self.text(col)) < tfloat(o1.text(col))
        else:
            return QTreeWidgetItem.__lt__(self, o1)


#set buffer file
bufferpath = os.path.join(orngEnviron.directoryNames["bufferDir"], "pipa")
try:
    os.makedirs(bufferpath)
except:
    pass
bufferfile = os.path.join(bufferpath, "database.sq3")

class ListItemDelegate(QStyledItemDelegate):
    def sizeHint(self, option, index):
        size = QStyledItemDelegate.sizeHint(self, option, index)
        size = QSize(size.width(), size.height() + 4)
        return size

    def createEditor(self, parent, option, index):
        return QLineEdit(parent)
    
    def setEditorData(self, editor, index):
        editor.setText(index.data(Qt.DisplayRole).toString())
        
    def setModelData(self, editor, model, index):
#        print index
        model.setData(index, QVariant(editor.text()), Qt.EditRole)

class SelectionSetsWidget(QFrame):
    """ Widget for managing multiple stored item selections 
    """
    def __init__(self, parent):
        QFrame.__init__(self, parent)
        self.setContentsMargins(0, 0, 0, 0)
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(1)
#        self._titleLabel = QLabel(self)
#        self._titleLabel
#        layout.addWidget(self._titleLabel)
        self._setNameLineEdit = QLineEdit(self)
        layout.addWidget(self._setNameLineEdit)
        
        self._setListView = QListView(self)
        self._listModel = QStandardItemModel(self)
        self._proxyModel = QSortFilterProxyModel(self)
        self._proxyModel.setSourceModel(self._listModel)
        
        self._setListView.setModel(self._proxyModel)
        self._setListView.setItemDelegate(ListItemDelegate(self))
        
        self.connect(self._setNameLineEdit, SIGNAL("textChanged(QString)"), self._proxyModel.setFilterFixedString)
        
        self._completer = QCompleter(self._listModel, self)
        
        self._setNameLineEdit.setCompleter(self._completer)
        
        self.connect(self._listModel, SIGNAL("itemChanged(QStandardItem *)"), self._onSetNameChange)
        layout.addWidget(self._setListView)
        buttonLayout = QHBoxLayout()
        
        self._addAction = QAction("+", self)
        self._updateAction = QAction("Update", self)
        self._removeAction = QAction("-", self)
        
        self._addToolButton = QToolButton(self)
        self._updateToolButton = QToolButton(self)
        self._removeToolButton = QToolButton(self)
        self._updateToolButton.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Minimum)
        
        self._addToolButton.setDefaultAction(self._addAction)
        self._updateToolButton.setDefaultAction(self._updateAction)
        self._removeToolButton.setDefaultAction(self._removeAction)
         
        buttonLayout.addWidget(self._addToolButton)
        buttonLayout.addWidget(self._updateToolButton)
        buttonLayout.addWidget(self._removeToolButton)
        
        layout.addLayout(buttonLayout)
        self.setLayout(layout)
        
        self.connect(self._addAction, SIGNAL("triggered()"), self.addCurrentSelection)
        self.connect(self._updateAction, SIGNAL("triggered()"), self.updateSelectedSelection)
        self.connect(self._removeAction, SIGNAL("triggered()"), self.removeSelectedSelection)
        
        self.connect(self._setListView.selectionModel(), SIGNAL("selectionChanged(QItemSelection, QItemSelection)"), self._onListViewSelectionChanged)
        self.selectionModel = None
        self._selections = []
        
    def sizeHint(self):
        size = QFrame.sizeHint(self)
        return QSize(size.width(), 200)
        
    def _onSelectionChanged(self, selected, deselected):
        self.setSelectionModified(True)
        
    def _onListViewSelectionChanged(self, selected, deselected):
        try:
            index= self._setListView.selectedIndexes()[0]
        except IndexError:
            return 
        self.commitSelection(self._proxyModel.mapToSource(index).row())

    def _onSetNameChange(self, item):
        self.selections[item.row()].name = str(item.text())
                
    def _setButtonStates(self, val):
        self._updateToolButton.setEnabled(val)
        
    def setSelectionModel(self, selectionModel):
        if self.selectionModel:
            self.disconnect(self.selectionModel, SIGNAL("selectionChanged(QItemSelection, QItemSelection)", self._onSelectionChanged))
        self.selectionModel = selectionModel
        self.connect(self.selectionModel, SIGNAL("selectionChanged(QItemSelection, QItemSelection)"), self._onSelectionChanged)
        
    def addCurrentSelection(self):
        item = self.addSelection(SelectionByKey(self.selectionModel.selection(), name="New selection", key=(1, 2, 3 ,10)))
        index = self._proxyModel.mapFromSource(item.index())
        self._setListView.setCurrentIndex(index)
        self._setListView.edit(index)
        self.setSelectionModified(False)
    
    def removeSelectedSelection(self):
        i = self._proxyModel.mapToSource(self._setListView.currentIndex()).row()
        self._listModel.takeRow(i)
        del self.selections[i]
    
    def updateCurentSelection(self):
        i = self._proxyModel.mapToSource(self._setListView.selectedIndex()).row()
        self.selections[i].setSelection(self.selectionModel.selection())
        self.setSelectionModified(False)
        
    def addSelection(self, selection, name=""):
        self._selections.append(selection)
        item = QStandardItem(selection.name)
        item.setFlags(item.flags() ^ Qt.ItemIsDropEnabled)
        self._listModel.appendRow(item)
        self.setSelectionModified(False)
        return item
        
    def updateSelectedSelection(self):
        i = self._proxyModel.mapToSource(self._setListView.currentIndex()).row()
        self.selections[i].setSelection(self.selectionModel.selection())
        self.setSelectionModified(False)
        
    def setSelectionModified(self, val):
        self._selectionModified = val
        self._setButtonStates(val)
        self.emit(SIGNAL("selectionModified(bool)"), bool(val))
        
    def commitSelection(self, index):
        selection = self.selections[index]
        selection.select(self.selectionModel)
        
    def setSelections(self, selections):
        self._listModel.clear()
#        print selections
        for selection in selections:
            self.addSelection(selection)
            
    def selections(self):
        return self._selections
    
    selections = property(selections, setSelections)
    
class SelectionByKey(object):
    """ An object stores item selection by unique key values 
    (works only for row selections in list and table models)
    Example::
        ## Save selection by unique tuple pairs (DisplayRole of column 1 and 2)
        selection = SelectionsByKey(itemView.selectionModel().selection(), key = (1,2))
        ...
        ## restore selection (Possibly omitting rows not present in the model)  
        selection.select(itemView.selectionModel())
    """
    
    def __init__(self, itemSelection, name="", key=(0,)):
        self._key = key
        self.name = name
        self._selected_keys = []
        if itemSelection:
            self.setSelection(itemSelection)
        
    def _row_key(self, model, row):
        val = lambda row, col: str(model.data(model.index(row, col), Qt.DisplayRole).toString())
        return tuple(val(row, col) for col in self._key)
    
    def setSelection(self, itemSelection):
        self._selected_keys = [self._row_key(ind.model(), ind.row()) for ind in itemSelection.indexes() if ind.column() == 0]
    
    def select(self, selectionModel):
        model = selectionModel.model()
        selected = []
        selectionModel.clear()
        for i in range(model.rowCount()):
            if self._row_key(model, i) in self._selected_keys:
                selectionModel.select(model.index(i, 0), QItemSelectionModel.Select | QItemSelectionModel.Rows)

    def __len__(self):
        return len(self._selected_keys)
                
class SortedListWidget(QWidget):
    class _MyItemDelegate(QStyledItemDelegate):
        def __init__(self, sortingModel, parent):
            QStyledItemDelegate.__init__(self, parent)
            self.sortingModel = sortingModel
            
        def sizeHint(self, option, index):
            size = QStyledItemDelegate.sizeHint(self, option, index)
            return QSize(size.width(), size.height() + 4)
            
        def createEditor(self, parent, option, index):
            cb = QComboBox(parent)
            cb.setModel(self.sortingModel)
            cb.showPopup()
            return cb
        
        def setEditorData(self, editor, index):
            pass # TODO: sensible default 
        
        def setModelData(self, editor, model, index):
            text = editor.currentText()
            model.setData(index, QVariant(text))
    
    def __init__(self, *args):
        QWidget.__init__(self, *args)
        self.setContentsMargins(0, 0, 0, 0)
        gridLayout = QGridLayout()
        gridLayout.setContentsMargins(0, 0, 0, 0)
        gridLayout.setSpacing(1)
        self._listView = QListView(self)
        self._listView.setModel(QStandardItemModel(self))
#        self._listView.setDragEnabled(True)
        self._listView.setDropIndicatorShown(True)
        self._listView.viewport().setAcceptDrops(True)
        self._listView.setDragDropMode(QAbstractItemView.InternalMove)
        self._listView.setMinimumHeight(100)
        
        gridLayout.addWidget(self._listView, 0, 0, 2, 2)
        
        vButtonLayout = QVBoxLayout()
        
        self._upAction = QAction(QIcon(os.path.join(orngEnviron.widgetDir, "icons/Dlg_up3.png")), "Up", self)
        
        self._upButton = QToolButton(self)
        self._upButton.setDefaultAction(self._upAction)
        self._upButton.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)

        self._downAction = QAction(QIcon(os.path.join(orngEnviron.widgetDir, "icons/Dlg_down3.png")), "Down", self)
        self._downButton = QToolButton(self)
        self._downButton.setDefaultAction(self._downAction)
        self._downButton.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.MinimumExpanding)
        
        vButtonLayout.addWidget(self._upButton)
        vButtonLayout.addWidget(self._downButton)
        
        gridLayout.addLayout(vButtonLayout, 0, 2, 2, 1)
        
        hButtonLayout = QHBoxLayout()
        
        self._addAction = QAction("+", self)
        self._addButton = QToolButton(self)
        self._addButton.setDefaultAction(self._addAction)
        
        self._removeAction = QAction("-", self)
        self._removeButton = QToolButton(self)
        self._removeButton.setDefaultAction(self._removeAction)
        hButtonLayout.addWidget(self._addButton)
        hButtonLayout.addWidget(self._removeButton)
        hButtonLayout.addStretch(10)
        gridLayout.addLayout(hButtonLayout, 2, 0, 1, 2)
        
        self.setLayout(gridLayout)
        
        self.connect(self._addAction, SIGNAL("triggered()"), self._onAddAction)
        self.connect(self._removeAction, SIGNAL("triggered()"), self._onRemoveAction)
        self.connect(self._upAction, SIGNAL("triggered()"), self._onUpAction)
        self.connect(self._downAction, SIGNAL("triggered()"), self._onDownAction)
        
    def sizeHint(self):
        size = QWidget.sizeHint(self)
        return QSize(size.width(), 100)
        
    def _onAddAction(self):
        item = QStandardItem("")
        item.setFlags(item.flags() ^ Qt.ItemIsDropEnabled)
        self._listView.model().appendRow(item)
        self._listView.setCurrentIndex(item.index())
        self._listView.edit(item.index()) 
    
    def _onRemoveAction(self):
        current = self._listView.currentIndex()
        self._listView.model().takeRow(current.row())
    
    def _onUpAction(self):
        row = self._listView.currentIndex().row()
        model = self._listView.model()
        if row > 0:
            items = model.takeRow(row)
            model.insertRow(row - 1, items)
            self._listView.setCurrentIndex(model.index(row - 1, 0))
    
    def _onDownAction(self):
        row = self._listView.currentIndex().row()
        model = self._listView.model()
        if row < model.rowCount() and row >= 0:
            items = model.takeRow(row)
            if row == model.rowCount():
                model.appendRow(items)
            else:
                model.insertRow(row + 1, items)
            self._listView.setCurrentIndex(model.index(row + 1, 0))
    
    def setModel(self, model):
        """ Set a model to select items from
        """
        self._model  = model
        self._listView.setItemDelegate(self._MyItemDelegate(self._model, self))
        
    def addItem(self, *args):
        """ Add a new entry in the list 
        """
        item = QStandardItem(*args)
        item.setFlags(item.flags() ^ Qt.ItemIsDropEnabled)
        self._listView.model().appendRow(item)
        
    def setItems(self, items):
        self._listView.model().clear()
        for item in items:
            self.addItem(item)
         
    def items(self):
        order = []
        for row in range(self._listView.model().rowCount()):
             order.append(str(self._listView.model().item(row ,0).text()))
        return order
    
    sortingOrder = property(items, setItems)
        
    
class OWPIPA(OWWidget):
    settingsList = [ "platform", "selectedExperiments", "server", "buffertime", "excludeconstant", "username", "password","joinreplicates",
                     "selectionSetsWidget.selections", "columnsSortingWidget.sortingOrder", "currentSelection", "log2", "raw", "experimentsHeaderState", "ctypei"]
    def __init__(self, parent=None, signalManager=None, name="PIPA"):
        OWWidget.__init__(self, parent, signalManager, name)
        self.outputs = [("Example table", ExampleTable)]

        self.platform = None
        self.username = ""
        self.password = ""
        self.log2 = False
        self.ctypei = 0

        self.selectedExperiments = []
        self.buffer = obiDicty.BufferSQLite(bufferfile)

        self.searchString = ""
        self.excludeconstant = False
        self.joinreplicates = False
        self.currentSelection = None
        
        self.experimentsHeaderState = {"": False, "Name":False, "Species": False, "Strain": False, "Experiment":False, "Genotype": False, "Treatment": False,
                                       "Growth": False, "Timepoint": False, "Replicate": False, "ID": False}

        self.exTypes  = []
        self.annots = {}

        self.controlArea.setMaximumWidth(250)
        self.controlArea.setMinimumWidth(250)

        OWGUI.button(self.controlArea, self, "Reload", callback=self.Reload)
        OWGUI.button(self.controlArea, self, "Clear cache", callback=self.clear_cache)
        
        b = OWGUI.widgetBox(self.controlArea, "Experiment Sets")
        self.selectionSetsWidget = SelectionSetsWidget(self)
        self.selectionSetsWidget.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)
#        self.controlArea.layout().addWidget(self.selectionSetsWidget)
        b.layout().addWidget(self.selectionSetsWidget)
        
        b = OWGUI.widgetBox(self.controlArea, "Sort output columns")
        self.columnsSortingWidget = SortedListWidget(self)
        self.columnsSortingWidget.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)
#        self.controlArea.layout().addWidget(self.columnsSortingWidget)
        b.layout().addWidget(self.columnsSortingWidget)
        self.columnsSortingWidget.setModel(QStringListModel(["Strain", "Experiment", "Genotype", "Timepoint", "Growth", "Species", "Id", "Name"]))
        self.columnsSortingWidget.sortingOrder = ["Strain", "Experiment", "Genotype", "Timepoint"]
        OWGUI.rubber(self.controlArea)

        box = OWGUI.widgetBox(self.controlArea, 'Expression Type')
        self.expressionTypesCB = cb = OWGUI.comboBox(box, self, "ctypei", items=[], debuggingEnabled=0, callback=self.UpdateCached)


        OWGUI.checkBox(self.controlArea, self, "excludeconstant", "Exclude labels with constant values" )
        OWGUI.checkBox(self.controlArea, self, "joinreplicates", "Average replicates (use median)" )
        OWGUI.checkBox(self.controlArea, self, "log2", "Logarithmic (base 2) transformation" )

        self.commit_button = OWGUI.button(self.controlArea, self, "&Commit", callback=self.Commit)
        self.commit_button.setDisabled(True)

        OWGUI.rubber(self.controlArea)
        OWGUI.rubber(self.controlArea)
        OWGUI.rubber(self.controlArea)
        OWGUI.rubber(self.controlArea)

        box  = OWGUI.widgetBox(self.controlArea, "Authentication")

        OWGUI.lineEdit(box, self, "username", "Username:", labelWidth=100, orientation='horizontal', callback=self.AuthChanged)
        self.passf = OWGUI.lineEdit(box, self, "password", "Password:", labelWidth=100, orientation='horizontal', callback=self.AuthChanged)
        self.passf.setEchoMode(QLineEdit.Password)

        OWGUI.lineEdit(self.mainArea, self, "searchString", "Search", callbackOnType=True, callback=self.SearchUpdate)
        self.headerLabels = ["", "Name", "Species", "Strain", "Experiment", "Genotype", "Treatment", "Growth", "Timepoint", "Replicate", "ID",
                             "Date RNA", "Adapter", "Who", "Date Rep", "Band", "Amount", "Experimenter","Polya", "Primer", "Shearing", "Unit"]
        self.experimentsWidget = QTreeWidget()
        self.experimentsWidget.setHeaderLabels(self.headerLabels)
        self.experimentsWidget.setSelectionMode(QTreeWidget.ExtendedSelection)
        self.experimentsWidget.setRootIsDecorated(False)
        self.experimentsWidget.setSortingEnabled(True)
        contextEventFilter = OWGUI.VisibleHeaderSectionContextEventFilter(self.experimentsWidget, self.experimentsWidget)
        self.experimentsWidget.header().installEventFilter(contextEventFilter)
        self.experimentsWidget.setItemDelegateForColumn(0, OWGUI.IndicatorItemDelegate(self, role=Qt.DisplayRole))
        self.experimentsWidget.setAlternatingRowColors(True)
        
        self.connect(self.experimentsWidget.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                     self.onSelectionChanged)

        self.selectionSetsWidget.setSelectionModel(self.experimentsWidget.selectionModel())

        self.mainArea.layout().addWidget(self.experimentsWidget)

        self.loadSettings()
        
        self.restoreHeaderState()
        
        self.connect(self.experimentsWidget.header(), SIGNAL("geometriesChanged()"), self.saveHeaderState)
        
        self.dbc = None

        self.AuthSet()

        QTimer.singleShot(100, self.UpdateExperiments)

        self.resize(800, 600)        

    def __updateSelectionList(self, oldList, oldSelection, newList):
        oldList = [oldList[i] for i in oldSelection]
        return [ i for i, new in enumerate(newList) if new in oldList]
    
    def AuthSet(self):
        if len(self.username):
            self.passf.setDisabled(False)
        else:
            self.passf.setDisabled(True)

    def AuthChanged(self):
        self.AuthSet()
        self.ConnectAndUpdate()

    def ConnectAndUpdate(self):
        self.Connect()
        self.UpdateExperiments(reload=True)

    def Connect(self):

        self.error(1)
        self.warning(1)

        def en(x):
            return x if len(x) else None

        self.dbc = obiDicty.PIPA(buffer=self.buffer, username=en(self.username), password=self.password)

        #check password
        if en(self.username) != None:
            try:
                self.dbc.annotations(reload=True)
            except obiDicty.AuthenticationError:
                self.error(1, "Wrong username or password")
                self.dbc = None
            except Exception, ex:
                print "Error when contacting the PIPA database", ex
                import traceback
                print traceback.format_exc()
                try: #mable cached?
                    self.dbc.annotations()
                    self.warning(1, "Can not access database - using cached data.")
                except Exception,ex:
                    self.dbc = None
                    self.error(1, "Can not access database.")

    def Reload(self):
        #self.buffer.clear()
        self.UpdateExperiments(reload=True)

    def clear_cache(self):
        self.buffer.clear()
        self.Reload()

    def ctype(self):
        """ Returns selected experiment type """
        if self.exTypes:
            return self.exTypes[self.ctypei][0]
        else:
            return "-1"

    def UpdateExperimentTypes(self):
        self.signalManager.freeze(self).push() #setFreeze(1)
        try:
            self.expressionTypesCB.clear()
            items = [desc for _,desc,_ in self.exTypes]
            self.expressionTypesCB.addItems(items)
        except IOError:
            pass
        finally:
            self.signalManager.freeze(self).pop() #setFreeze(0)
        self.ctypei = max(0, min(self.ctypei, len(self.exTypes)-1))

    def UpdateExperiments(self, reload=False):

        self.chipsl = []
        self.experimentsWidget.clear()
        self.items = []

        self.progressBarInit()

        if not self.dbc:
            self.Connect()
 
        annots = {}
        exTypes = []
        
        sucind = False #success indicator for database index

        try:
            annots = self.dbc.annotations(chips, reload=reload)
            exTypes = self.dbc.gene_expression_types(reload=reload)
            sucind = True
        except Exception, ex:
            try:
                annots = self.dbc.annotations()
                exTypes = self.dbc.gene_expression_types()
                self.warning(0, "Can not access database - using cached data.")
                sucind = True
            except Exception,ex:
                self.error(0, "Can not access database.")

        if sucind:
            self.warning(0)
            self.error(0)

        self.annots = annots
        self.exTypes = exTypes

        self.UpdateExperimentTypes() 

        elements = []
        pos = 0

        for chip,annot in self.annots.items():
            pos += 1
            d = defaultdict(lambda: "?", annot)
            elements.append(["", d["name"], d["species"], d["strain"], d["Experiment"], d["genotype"], d["treatment"], d["growth"], d["tp"], d["replicate"], chip] + \
                             [d[label.lower().replace(" ", "_")] for label in ["Date RNA", "Adapter", "Who", "Date Rep", "Band", "Amount", "Experimenter", "Polya", "Primer", "Shearing", "Unit"]])
            
#            self.progressBarSet((100.0 * pos) / len(chips))
            
            el = elements[-1]
            ci = MyTreeWidgetItem(self.experimentsWidget, el)

            self.items.append(ci)

        for i in range(len(self.headerLabels)):
            self.experimentsWidget.resizeColumnToContents(i)

        #which is the ok buffer version
        self.wantbufver = lambda x,ad=self.annots: defaultdict(lambda: "?", ad[x])["date"] #FIXME what attribute to use for version?

        self.UpdateCached()

        self.progressBarFinished()
        
        if self.currentSelection:
            self.currentSelection.select(self.experimentsWidget.selectionModel())

        self.handle_commit_button()

    def UpdateCached(self):
        if self.wantbufver and self.dbc:
            fn = self.dbc.chips_keynaming(self.ctype())

            for item in self.items:
                c = str(item.text(10))
                item.setData(0, Qt.DisplayRole, QVariant(" " if self.dbc.inBuffer(fn(c)) == self.wantbufver(c) else ""))

    def SearchUpdate(self, string=""):
        for item in self.items:
            item.setHidden(not all(s in item for s in self.searchString.split()))

    def Commit(self):
        if not self.dbc:
            self.Connect()
        allTables = []

        import time
        start = time.time()

        pb = OWGUI.ProgressBar(self, iterations=1000)

        table = None

        ids = []
        for item in self.experimentsWidget.selectedItems():
            ids += [ str(item.text(10)) ]

        transfn = None
        if self.log2:
            transfn = lambda x: math.log(x+1.0, 2)
        
        keys = {"Strain": "strain", "Genotype": "genotype", "Treatment": "treatment", "Timepoint": "tp", "Growth": "growth", "Replicate": "replicate", 
                "Species": "species", "Name": "name", "Date RNA": "date_rna", "Adapter":"adapter",
                "Who": "who", "Date Rep": "date_rep", "Band": "band", "Amount": "amount", "Experimenter":"experimenter",
                "Polya": "polya", "Primer": "primer", "Shearing": "shearing", "Unit": "unit", "Experiment":"Experiment"}
        
        hview = self.experimentsWidget.header()
        shownHeaders = [label for i, label in list(enumerate(self.headerLabels))[1:] if not hview.isSectionHidden(i)]
        allowed_labels = [keys.get(label, label) for label in shownHeaders]
        
        if self.joinreplicates and "id" not in allowed_labels:
            # need 'id' labels in join_replicates for attribute names
            allowed_labels.append("id")
        
        if len(ids):
            table = self.dbc.get_data(ids=ids, callback=pb.advance, exclude_constant_labels=self.excludeconstant, bufver=self.wantbufver, transform=transfn, allowed_labels=allowed_labels, ctype=self.ctype())

            if self.joinreplicates:
                table = obiDicty.join_replicates(table, ignorenames=["replicate", "id", "name", "map_stop1"], namefn=None, avg=obiDicty.median)

            end = int(time.time()-start)
            

            # Sort attributes
            sortOrder = self.columnsSortingWidget.sortingOrder
            # print sortOrder
            
            attributes = sorted(table.domain.attributes, key=lambda attr: tuple([attr.attributes.get(keys[name], "") for name in sortOrder]))
            domain = orange.Domain(attributes, table.domain.classVar)
            domain.addmetas(table.domain.getmetas())
            table = orange.ExampleTable(domain, table)
            
            from Orange.orng.orngDataCaching import data_hints
            data_hints.set_hint(table, "taxid", "352472") 
            data_hints.set_hint(table, "genesinrows", False)
            
            self.send("Example table", table)

            self.UpdateCached()

        pb.finish()

    def onSelectionChanged(self, selected, deselected):
        self.handle_commit_button()
        
    def handle_commit_button(self):
        self.currentSelection = SelectionByKey(self.experimentsWidget.selectionModel().selection(),
                                               key=(1, 2, 3, 10))
        self.commit_button.setDisabled(not len(self.currentSelection))

    def saveHeaderState(self):
        hview = self.experimentsWidget.header()
        for i, label in enumerate(self.headerLabels):
            self.experimentsHeaderState[label] = hview.isSectionHidden(i)
            
    def restoreHeaderState(self):
        hview = self.experimentsWidget.header()
        for i, label in enumerate(self.headerLabels): 
            hview.setSectionHidden(i, self.experimentsHeaderState.get(label, True))
            self.experimentsWidget.resizeColumnToContents(i)
            
if __name__ == "__main__":
    app  = QApplication(sys.argv)
##    from pywin.debugger import set_trace
##    set_trace()
    obiDicty.verbose = True
    w = OWPIPA()
    w.show()
    app.exec_()
    w.saveSettings()
            
        



"""A library for searching frequent molecular fragments (substructures) based on the
Mining Molecular Fragments: Finding Relevant Substructures of Molecules
Christian Borgelt and Michael R. Berthold.
Classes (see their corresponding __doc__ strings for further detail):
    Fragment        : Representation of the fragment
    FragmentMiner   : The main class that does the search
    Fragmenter      : A class that is used to fragment an ExampleTable
    FragmentBasedLearner    : A learner wrapper class that first runs the molecular fragmentation on the data
"""
try:
    from openbabel import OBMol, OBAtom, OBBond, OBSmartsPattern, OBConversion, OBMolAtomIter, OBMolBondIter, OBAtomBondIter
except ImportError:
    pass
##from pybel import *
from copy import deepcopy
import orange
import sys, os


debug=False

class Atom(object):
    def __init__(self, molecule, atomicNum=6, aromaticFlag=False, atomIndex=0):
        self.molecule=molecule
        self.atomicNum=atomicNum
        self.aromaticFlag=aromaticFlag
        self.atomIndex=atomIndex
        self.extendedIndex=0
        self.lastExtendedBondOrder=1
        self.lastExtendedAtomicNum=1
        self.molecule.AddAtom(self)
    def GetAtomicNum(self): return self.atomicNum
    def IsAromatic(self): return self.aromaticFlag
    def Match(self, atom):
        #Match this atom to an OBAtom
        if self.atomicNum==atom.GetAtomicNum() and self.IsAromatic()==atom.IsAromatic():
            return True
        else:
            return False
    def GetIdx(self):
        return self.atomIndex

class Bond(object):
    def __init__(self, molecule, atom1=None, atom2=None, bondOrder=1):
        self.molecule=molecule
        self.atom1=atom1
        self.atom2=atom2
        self.bondOrder=bondOrder
        self.molecule.AddBond(self)
    def GetBondOrder(self): return self.bondOrder
    def GetNbrAtom(self, atom):
        if atom==self.atom1:
            return self.atom2
        elif atom==self. atom2:
            return  self.atom1
        else:
            raise Exception("Atom does not belong to this bond")
    def Match(self, bond):
        #Match this bond to an OBBond
        if self.GetbondOrder()==bond.GetBondOrder():
            return True
        else:
            return False

class Ring(object):
    def __init__(self, molecule, obRing, embeding):
        self.molecule=molecule
        molecule.AddRing(self)
        self.ringAtoms=[]
        self.isAromatic=obRing.IsAromatic()
        reverseEmbedingDict=embeding.GetReverseDict()
        newAtoms={}
        self.extendedIndex=oldNextExtendedIndex=molecule.nextExtendedIndex
        for ind1, ind2 in GetRingPairs(obRing._path):
            atom1=embeding.GetReverseEmbededAtom(ind1, None) or newAtoms.get(ind1, None)
            if not atom1:
                atom1=Atom(molecule, embeding.molecule.GetAtom(ind1).GetAtomicNum(), obRing.IsAromatic())
                newAtoms[ind1]=atom1
                embeding[atom1.atomIndex]=ind1
            atom2=embeding.GetReverseEmbededAtom(ind2, None) or newAtoms.get(ind2, None)
            if not atom2:
                atom2=Atom(molecule, embeding.molecule.GetAtom(ind2).GetAtomicNum(), obRing.IsAromatic())
                newAtoms[ind2]=atom2
                embeding[atom2.atomIndex]=ind2
            Bond(molecule, atom1, atom2, embeding.molecule.GetBond(ind1, ind2).GetBondOrder())
            self.ringAtoms.append(atom1)
        for atom in newAtoms.values():
            atom.extendedIndex=oldNextExtendedIndex
        self.extendedIndex=oldNextExtendedIndex
        molecule.nextExtendedIndex=oldNextExtendedIndex+1
    def Size(self):
        return len(self.ringAtoms)
    def __getattr__(self, name):
        if name == "_path":
            return [a.GetIdx() for a in self.ringAtoms]
        raise AttributeError 

class Molecule(object):
    def __init__(self):
        self.atoms=[]
        self.bonds=[]
        self.rings=[]
        self.nextExtendedIndex=0

    def __deepcopy__(self, memo):
        mol=Molecule()
        memo[id(self)]=mol
        mol.__in_place_deepcopy__(self, memo)
        return mol
    def __in_place_deepcopy__(self, mol, memo):
        if debug: print "Copying molecule"
        self.atoms=deepcopy(mol.atoms, memo)
        #memo[mol.atoms]=self.atoms
        self.bonds=deepcopy(mol.bonds, memo)
        #memo[mol.bonds]=self.bonds
        self.rings=deepcopy(mol.rings, memo)
        self.nextExtendedIndex=mol.nextExtendedIndex
        #memo[mol.rings]=self.rings"""
        
    def AddAtom(self, atom):
        if atom in self.atoms:
            raise Exception("Atom already present")
        self.atoms.append(atom)
        atom.atomIndex=len(self.atoms)-1
        atom.extendedIndex=self.nextExtendedIndex
        self.nextExtendedIndex+=1
    def AddBond(self, bond):
        if bond in self.bonds:
            raise Exception("Bond already present")
        self.bonds.append(bond)
        bond.bondIndex=len(self.bonds)-1
    def AddRing(self, ring):
        if ring in self.rings:
            raise Exception("Ring already present")
        self.rings.append(ring)
        ring.ringIndex=len(self.rings)-1
    def GetBond(self, atom1, atom2):
        for bond in self.bonds:
            if bond.atom1==atom1 and bond.atom2==atom2:
                return bond
            if bond.atom1==atom2 and bond.atom2==atom1:
                return bond
        return None
    def GetAtom(self, index):
        return self.atoms[index]
    
def cmpAtoms(atom1, atom2):
    return atom1.GetAtomicNum()==atom2.GetAtomicNum() and atom1.IsAromatic()==atom2.IsAromatic()
def cmpAtomBonds(bond1, bond2):
    return bond1.IsAromatic() and bond2.IsAromatic() or bond1.GetBondOrder()==bond2.GetBondOrder()

class Embeding(dict):
    def __init__(self, embeding={}, molecule=None, fragment=None):
        dict.__init__(self, embeding)
        self.molecule=molecule
        self.fragment=fragment
        if embeding.__class__==Embeding:
            if not self.molecule:
                self.molecule=embeding.molecule
            if not self.fragment:
                self.fragment=embeding.fragment
    def __deepcopy__(self, memo):
        if id(self.fragment) in memo:
            return Embeding(self, fragment=memo[id(self.fragment)])
        else:
            return Embeding(self)
    def GetEmbededAtom(self, atomInd):
        return self.molecule.GetAtom(self.__getitem__(atomInd))
    def GetReverseEmbededAtom(self, atomInd, default=None):
        rev=self.GetReverseDict()
        if atomInd in rev:
            return self.fragment.atoms[rev[atomInd]]
        else:
            return default
    def GetReverseDict(self):
        return dict([(value, key) for key, value in self.items()])

def GetRingPairs(list):
    return reduce(lambda a,b:a+[(a[-1][1],b)], list[1:], [(list[-1],list[0])])

class FragmentExtension(object):
    def __init__(self, startAtomInd, embedings):
        self.startAtomInd=startAtomInd
        self.embedings=embedings
    def IsEquivalent(self, extension):
        pass
    def MergeFrom(self, extension):
        pass
    def Extend(self, fragment):
        pass

class FragmentExtensionByAtom(FragmentExtension):
    def __init__(self, atomicNum, aromatic, atomIndices, embedings):
        self.atomicNum=atomicNum
        self.aromatic=aromatic
        self.atomIndices=atomIndices
        self.embedings=embedings
    def IsEquivalent(self, extension):
        return self.atomicNum==extension.atomicNum and self.aromatic==extension.aromatic
    def MergeFrom(self, extension):
        self.embedings.extend(extension.embedings)
        self.atomIndices.extend(extension.atomIndices)
    def Extend(self, fragment):
        atom=Atom(fragment, self.atomicNum, self.aromatic)
        embedings=[Embeding(e, fragment=fragment)for e in self.embedings]
        for embeding, atomInd in zip(embedings, self.atomIndices):
            embeding[atom.atomIndex]=atomInd
        fragment.embedings=embedings
        
class FragmentExtensionByBondAtom(FragmentExtension):
    def __init__(self, startAtomInd, bondOrder, endAtoms, embedings):
        FragmentExtension.__init__(self, startAtomInd, embedings)
        self.bondOrder=bondOrder
        self.endAtoms=endAtoms
    def IsEquivalent(self, extension):
        return self.startAtomInd==extension.startAtomInd and self.bondOrder==extension.bondOrder and cmpAtoms(self.endAtoms[0], extension.endAtoms[0])
    def MergeFrom(self, extension):
        self.endAtoms.extend(extension.endAtoms)
        self.embedings.extend(extension.embedings)
    def Extend(self, fragment):
        if debug: print "Extending by bond and atom"
        sAtom=fragment.atoms[self.startAtomInd]
        atom=Atom(fragment, self.endAtoms[0].GetAtomicNum(),self.endAtoms[0].IsAromatic())
        #atom.SetAtomicNum(self.endAtoms[0].GetAtomicNum())
        
        Bond(fragment, sAtom, atom, self.bondOrder)
        sAtom.lastExtendedBondOrder=self.bondOrder
        sAtom.lastExtendedAtomicNum=atom.GetAtomicNum()
        fragment.lastExtendedAtomIndex=sAtom.extendedIndex
        embedings=[Embeding(e, fragment=fragment)for e in self.embedings]
        for embeding, endAtom in zip(embedings, self.endAtoms):
            embeding[atom.atomIndex]=endAtom.GetIdx()
        fragment.embedings=embedings
        
class FragmentExtensionByBond(FragmentExtension):
    def __init__(self, startAtomInd, bondOrder, endAtomInd, embedings):
        FragmentExtension.__init__(self, startAtomInd, embedings)
        self.bondOrder=bondOrder
        self.endAtomInd=endAtomInd
    def IsEquivalent(self, extension):
        return self.startAtomInd==extension.startAtomInd and self.bondOrder==extension.bondOrder and self.endAtomInd==extension.endAtomInd
    def MergeFrom(self, extension):
        self.embedings.extend(extension.embedings)
    def Extend(self, fragment):
        if debug: print "Extending by bond"
        sAtom=fragment.atoms[self.startAtomInd]
        eAtom=fragment.atoms[self.endAtomInd]
        Bond(fragment, sAtom, eAtom, self.bondOrder)
        sAtom.lastExtendedBondOrder=self.bondOrder
        sAtom.lastExtendedAtomicNum=eAtom.GetAtomicNum()
        fragment.lastExtendedAtomIndex=sAtom.extendedIndex
        fragment.embedings=[Embeding(e, fragment=fragment) for e in self.embedings]

class FragmentExtensionByBondRing(FragmentExtension):
    def __init__(self, startAtomInd, bondOrder, endAtoms, rings, embedings):
        FragmentExtension.__init__(self, startAtomInd, embedings)
        self.bondOrder=bondOrder
        self.endAtoms=endAtoms
        self.rings=rings
    def GetRingMapping(self, endAtom1, ring1, embeding1, endAtom2, ring2, embeding2):
        rDict1=embeding1.GetReverseDict()
        rDict2=embeding2.GetReverseDict()
        ringMappings=[]
        i=list(ring1._path).index(endAtom1.GetIdx())
        c0=list(ring1._path)[i:]+list(ring1._path)[:i]
        i=list(ring2._path).index(endAtom2.GetIdx())
        c1=list(ring2._path)[i:]+list(ring2._path)[:i]
        l=list(c1[1:]) #l=list(c1[:-1])
        l.reverse()
        c2=[c1[0]]+l #c2=[c1[-1]]+l
        for c in [c1,c2]:
            mapping=[]
            for (ra11, ra12), (ra21, ra22) in zip(GetRingPairs(map(lambda i:ring1.GetParent().GetAtom(i), c0)),GetRingPairs(map(lambda i:ring2.GetParent().GetAtom(i), c))):
                if cmpAtoms(ra12, ra22) and cmpAtomBonds(ring1.GetParent().GetBond(ra11, ra12), ring2.GetParent().GetBond(ra21, ra22)):
                    mapping.append((ra12.GetIdx(), ra22.GetIdx()))
                else:
                    break
            else:
                ringMappings.append(dict(mapping))
        return ringMappings
    def IsEquivalent(self, extension):
        if self.startAtomInd==extension.startAtomInd and self.bondOrder==extension.bondOrder and len(self.rings[0]._path)==len(extension.rings[0]._path) \
           and self.rings[0].fingerprint==extension.rings[0].fingerprint:
            return bool(self.GetRingMapping(self.endAtoms[0], self.rings[0], self.embedings[0], extension.endAtoms[0], extension.rings[0], extension.embedings[0]))
        return False
    def MergeFrom(self, extension):
        self.endAtoms.extend(extension.endAtoms)
        self.embedings.extend(extension.embedings)
        self.rings.extend(extension.rings)
    def Extend(self, fragment):
        if debug: print "Extending by ring and bond"
        sAtom=fragment.atoms[self.startAtomInd]
        endAtom1=self.endAtoms[0]
        ring1=self.rings[0]    #One ring to rule them all
        embeding1=Embeding(self.embedings[0], fragment=fragment)
        #Add all the atoms and bonds in the ring
        newRing=Ring(fragment, ring1, embeding1) #Changes embeding
        Bond(fragment, fragment.atoms[self.startAtomInd], fragment.atoms[embeding1.GetReverseDict()[endAtom1.GetIdx()]], self.bondOrder)
        sAtom.lastExtendedAtomicNum=endAtom1.GetAtomicNum()
        sAtom.lastExtendedBondOrder=self.bondOrder
        fragment.lastExtendedAtomIndex=sAtom.extendedIndex
        #Update embedings
        #embeding1=self.embedings[0]
        fragment.embedings=[]
        for endAtom, ring, embeding in zip(self.endAtoms, self.rings, self.embedings):
            #embeding[sAtom.atomIndex]=endAtom.GetIdx()
            mappings=self.GetRingMapping(endAtom1, ring1, embeding1, endAtom, ring, embeding)
            for map in mappings:
                embeding=Embeding(embeding, fragment=fragment)
                for atom in newRing.ringAtoms:
                    embeding[atom.atomIndex]=map[embeding1[atom.atomIndex]]
                embeding[newRing]=ring
                fragment.embedings.append(embeding)
        
class FragmentExtensionByRing(FragmentExtension):
    def __init__(self, rings, embedings):
        FragmentExtension.__init__(self, 0, embedings)
        self.rings=rings
    def GetRingMapping(self, ring1, embeding1, ring2, embeding2):
        rDict1=embeding1.GetReverseDict()
        rDict2=embeding2.GetReverseDict()
        ringMappings=[]
        for i in range(ring2.Size()):
            c1=list(ring2._path)[i:]+list(ring2._path)[:i]
            l=list(c1[1:])
            l.reverse()
            c2=[c1[0]]+l
            for c in [c1,c2]:
                mapping=[]
                for (ra11, ra12), (ra21, ra22) in zip(GetRingPairs(map(lambda i:ring1.GetParent().GetAtom(i), ring1._path)),GetRingPairs(map(lambda i:ring2.GetParent().GetAtom(i), c))):
                    if cmpAtoms(ra12, ra22) and cmpAtomBonds(ring1.GetParent().GetBond(ra11, ra12), ring2.GetParent().GetBond(ra21, ra22)) and rDict1.get(ra12.GetIdx(), None)==rDict2.get(ra22.GetIdx(), None):
                        mapping.append((ra12.GetIdx(), ra22.GetIdx()))
                    else:
                        break
                else:
                    ringMappings.append(dict(mapping))
        return ringMappings
    def IsEquivalent(self, extension):
        if len(self.rings[0]._path)==len(extension.rings[0]._path) and self.rings[0].fingerprint==extension.rings[0].fingerprint:
            return bool(self.GetRingMapping(self.rings[0], self.embedings[0], extension.rings[0], extension.embedings[0]))
        return False
    def MergeFrom(self, extension):
        self.rings.extend(extension.rings)
        self.embedings.extend(extension.embedings)
    def Extend(self, fragment):
        if debug: print "Extending by ring"
        ring1=self.rings[0]
        tmpEmbeding=Embeding(self.embedings[0],fragment=fragment)
        newRing=Ring(fragment, ring1, tmpEmbeding) #Changes embeding
        embeding1=self.embedings[0]
##        fragment.lastExtendedAtomIndex=sAtom.extendedIndex
        fragment.embedings=[]
        for ring, embeding in zip(self.rings, self.embedings):
            mappings=self.GetRingMapping(ring1, embeding1, ring, embeding)
            for map in mappings:
                embeding=Embeding(embeding, fragment=fragment)
                for atom in newRing.ringAtoms:
                    embeding[atom.atomIndex]=map[tmpEmbeding[atom.atomIndex]]
                embeding[newRing]=ring
                fragment.embedings.append(embeding)
        
class Fragment(Molecule):
    """A class representing a molecular fragment
    Methods:
        ToOBMol()   : Returns an openbabel.OBMol object representation
        ToSmiles()  : Returns a SMILES code representation
        ToCanonicalSmiles() : Returns a canonical SMILES code representation
        Support()   : Returns the support of the fragment in the active set
        OcurrencesIn(smiles): Returns the number of times a fragment is containd
                    in the molecule represented by the smiles code argument
        ContainedIn(smiles) : Returns True if the fragment is present in the molecule
                    represented by the smiles code argument
    """
    def __init__(self, miner=None, excludeAtomList=[]):
        Molecule.__init__(self)
        self.embedings=[]
        self.miner=miner
        self.excludeAtomList=excludeAtomList
        self.lastExtendedAtomicNum=0
        self.lastExtendedAtomIndex=0
        self.writer=OBConversion()
        self.writer.SetInAndOutFormats("smi","smi")

    def __deepcopy__(self, memo):
        f=Fragment()
        memo[id(self)]=f
        Molecule.__in_place_deepcopy__(f, self, memo)
        f.embedings=[Embeding(e,fragment=f) for e in self.embedings]
        f.miner=self.miner
        f.excludeAtomList=self.excludeAtomList
        f.lastExtendedAtomIndex=self.lastExtendedAtomIndex
        return f
        
    def InitializeFragment(self, atomicNum):
        mol=OBMol()
        atom=Atom(self, atomicNum)
        for mol in self.miner.GetAllMolecules():
            for a in OBMolAtomIter(mol):
                if atom.Match(a):
                    self.embedings.append(Embeding({atom.atomIndex : a.GetIdx()}, molecule=mol, fragment=self))
                    
    def IsAtomExcluded(self, atom):
        return atom.GetAtomicNum() in self.excludeAtomList
    
    def GetCandidateAtoms(self):
        return filter(lambda a:a.extendedIndex>=self.lastExtendedAtomIndex, self.atoms)

    def GetCandidateRings(self):
        return filter(lambda r:r.extendedIndex>=self.lastExtendedAtomIndex, self.rings)

    def FilterRings(self, candidateRings, embeding):
        for ring1 in candidateRings:
            for ring2 in filter(lambda r:type(r)==type(ring1), embeding.values()):
                if ring1.this==ring2.this:
                    break
            else:
                yield ring1
        
    def GetCandidatesFromAtom(self, atomInd, embeding):
        candidates=[]
        reverseEmbedingDict=embeding.GetReverseDict()
        atom=embeding.GetEmbededAtom(atomInd)
        if atom.GetParent().NumHvyAtoms()==1:
            return candidates
        for bond in OBAtomBondIter(atom):   #crashes if the atom is alone in a molecule
            nbrAtom=bond.GetNbrAtom(atom)
            if self.IsAtomExcluded(nbrAtom):
                    continue
            if bond.GetBondOrder()>self.atoms[atomInd].lastExtendedBondOrder or (bond.GetBondOrder()==self.atoms[atomInd].lastExtendedBondOrder and nbrAtom.GetAtomicNum()>=self.atoms[atomInd].lastExtendedAtomicNum):
                if self.miner.addWholeRings: #Whole rings are added at the same time (no new bond can connect to an atom already in the embeding)
                    if nbrAtom.IsInRing():
                        if not bond.IsInRing(): #ring to ring extensions are handled in GetRingCandidatesFromRing 
                            for ring in self.FilterRings(self.miner.rings[embeding.molecule], embeding):
                                if ring.IsMember(nbrAtom):
                                    candidates.append(FragmentExtensionByBondRing(atomInd, bond.GetBondOrder(), [nbrAtom], [ring], [embeding]))
##                        elif len(self.atoms)==1: #
##                            for ring in self.FilterRings(self.miner.rings[embeding.molecule], embeding):
##                                if ring.IsMember(nbrAtom) and ring.IsMember(atom) and ring not in addedRings:
##                                    candidates.append(FragmentExtensionByRing([ring], [embeding]))
##                                    addedRings.append(ring)
                           
                    elif nbrAtom.GetIdx() not in reverseEmbedingDict:
                        candidates.append(FragmentExtensionByBondAtom(atomInd, bond.GetBondOrder(), [nbrAtom], [embeding]))                                
                else: 
                    if nbrAtom.GetIdx() in reverseEmbedingDict:
                        if not self.GetBond(self.GetAtom(atomInd), self.GetAtom(reverseEmbedingDict[nbrAtom.GetIdx()])) :
                            candidates.append(FragmentExtensionByBond(atomInd, bond.GetBondOrder(), reverseEmbedingDict[nbrAtom.GetIdx()], [embeding]))
                    else:
                        candidates.append(FragmentExtensionByBondAtom(atomInd, bond.GetBondOrder(), [nbrAtom], [embeding]))
        #candidates.sort(lambda a,b:cmp(a[0],b[0]) or cmp(a[1].GetAtomicNum(),b[1].GetAtomicNum()))
        return candidates

    def GetRingCandidatesFromRing(self, ring, embeding):
        candidates=[]
        reverseEmbedingDict=embeding.GetReverseDict()
        for candidateRing in self.FilterRings(self.miner.rings[embeding.molecule], embeding):
            for atom in ring.ringAtoms:
                if candidateRing.IsMember(embeding.GetEmbededAtom(atom.atomIndex)):
                    candidates.append(FragmentExtensionByRing([candidateRing], [embeding]))
                    break
        return candidates                             
    
    def GetCandidatesFromEmbeding(self, embeding):
        candidates=[]
        if len(self.atoms)==1 and self.miner.addWholeRings:
            atom=embeding.GetEmbededAtom(self.atoms[0].GetIdx())
            if atom.IsInRing():
                for ring in embeding.molecule.rings:
                    if ring.IsMember(atom):
                        candidates.append(FragmentExtensionByRing([ring],[embeding]))
                return candidates
            
        if debug: print "Parsing candidate bonds"
        for atom in self.GetCandidateAtoms():
            candidates.extend(self.GetCandidatesFromAtom(atom.atomIndex, embeding))
        
        if debug: print "Parsing candidate rings"
        for ring in self.GetCandidateRings():
            candidates.extend(self.GetRingCandidatesFromRing(ring, embeding))            
        return candidates        
        
    def Extend(self):
        if debug :print "Extending"
        candidates=[]
        for embeding in self.embedings:
            c=self.GetCandidatesFromEmbeding(embeding)
            candidates.extend(c)
            #embedingDict[embeding]=c
            
        #group equivalent candidates
        if debug: print "Grouping candidates"
        groups=[]
        for extension in candidates:
            for ext in groups:
                if ext.__class__==extension.__class__ and ext.IsEquivalent(extension):
                    ext.MergeFrom(extension)
                    break
            else:
                groups.append(extension)
  
        #generate new fragments
        if debug: print "Generating new fragments"
        newFragments=[]
        for extension in groups:
            #set_trace()
            f=deepcopy(self)
            extension.Extend(f)
            newFragments.append(f)
        return newFragments
    
    def ToOBMol(self):
        atomCache={}
        mol=OBMol()
        mol.BeginModify()
        for sourceAtom in self.atoms:
            atom=mol.NewAtom()
            atom.SetAtomicNum(sourceAtom.GetAtomicNum())
            if sourceAtom.IsAromatic():
                atom.SetAromatic()
##                atom.SetSpinMultiplicity(2)
            atomCache[sourceAtom]=atom
        for sourceBond in self.bonds:
            mol.AddBond(atomCache[sourceBond.atom1].GetIdx(), atomCache[sourceBond.atom2].GetIdx(), sourceBond.GetBondOrder())
##        mol.SetAromaticPerceived()
        mol.AssignSpinMultiplicity()
##        mol.UnsetAromaticPerceived()
        mol.EndModify()
        return mol

    def ToSmiles(self):
        writer=OBConversion()
        writer.SetInAndOutFormats("smi", "smi")
        return writer.WriteString(self.ToOBMol()).strip()
    
    def ToCannonicalSmiles(self):
        atomCache={}
        mol=OBMol()
        for sourceAtom in self.atoms:
            atom=mol.NewAtom()
            atom.SetAtomicNum(sourceAtom.GetAtomicNum())
            if sourceAtom.IsAromatic():
                atom.SetAromatic()
                atom.SetSpinMultiplicity(2)
            atomCache[sourceAtom]=atom
        for sourceBond in self.bonds:
            mol.AddBond(atomCache[sourceBond.atom1].GetIdx(), atomCache[sourceBond.atom2].GetIdx(), sourceBond.GetBondOrder())
        writer=OBConversion()
        writer.SetInAndOutFormats("smi", "can")
        return writer.WriteString(mol).strip()

    def Support(self, activeSet=None):
        activeSet=self.miner.activeSet if activeSet==None else activeSet
        uniqueMolecules=set()
        for embeding in self.embedings:
            if embeding.molecule in activeSet:
                uniqueMolecules.add(embeding.molecule)
##        s=set(filter(lambda mol:self.ContainedIn(mol), self.miner.GetAllMolecules()))
##        if len(s) != len(uniqueMolecules):
##            writer=OBConversion()
##            writer.SetInAndOutFormats("smi", "smi")
##            print "\n",self.ToSmiles()
##            for m in uniqueMolecules: print writer.WriteString(m).strip()
##            for m in s: print writer.WriteString(m).strip()
        
        return float(len(uniqueMolecules))/float(len(activeSet) or 1)

    def OcurrencesIn(self, molecule):    
        pattern=OBSmartsPattern()
        pattern.Init(self.ToSmiles())
        return pattern.Match(molecule)
    
    def ContainedIn(self, molecule):
        return bool(self.OcurrencesIn(molecule))
            
class FragmentMiner(object):
    """A class for finding frequent molecular fragments
    Attributes:
        active      : list of smiles codes of active molecules
        inactive    : list of smiles codes of inactive molecules
        minSupport  : minimum frequency in the active set of the fragments to search for
        maxSupport  : maximum frequency in the inactive set of the fragments to search for
        addWholeRings : if True rings will be added as a whole rather then atom by atom
        canonicalPruning : if True a cache of all cannonical codes of all fragments will be kept to avoid
                    redundant search
        findClosed  : finds only fragments that are not sub-structures of any other fragment with the same support (default: True)
    Example:
    >>> miner = FragmentMiner(active = ["CC(C=N)=O", "c1ccccc1C=O", "SCC(N)O"], inactive = [], minSupport = 0.6)
    >>> for fragment in miner.Search():
    ... 	print fragment.ToSmiles() , "Support: %.3f" %fragment.Support()
    """
    def __init__(self, active, inactive=[], minSupport=0.2, maxSupport=0.2, addWholeRings=True, canonicalPruning=True, findClosed=True):
        self.active=filter(lambda m:m, map(self.LoadMolecules, active))
        self.inactive=filter(lambda m:m, map(self.LoadMolecules, inactive))
        self.minSupport=minSupport
        self.maxSupport=maxSupport
        self.rings={}
        self.atomCount={}
        self.findClosed=findClosed
        self.addWholeRings=addWholeRings
        self.canonicalPruning=canonicalPruning
        self.canonicalPruningSet={}
        self.loader=OBConversion()
        self.loader.SetInAndOutFormats("smi","smi")

    def LoadMolecules(self, smiles):
        mol=LoadMolFromSmiles(smiles)
##        if mol:
##            mol.StripSalts()
        return mol
        
    def GetAllMolecules(self):
        return self.active+self.inactive     

    def Initialize(self):
        """Initializes the search"""
        self.initialFragments=[]
        self.rings={}
        self.atomCount={}
        self.canonicalPruningSet={}
        candidates=[]
        ringCandidates=[]
        for mol in self.GetAllMolecules():
            mol.rings=self.rings[mol]=list(mol.GetSSSR())
            for ring in mol.rings:
                ring.fingerprint=set([mol.GetAtom(i).GetAtomicNum() for i in ring._path])
##            for ring in mol.rings:
##                candidates.append(FragmentExtensionByRing([ring],[Embeding(molecule=mol)]))
        
        for mol in self.GetAllMolecules():
            for atom in OBMolAtomIter(mol):
                self.atomCount[atom.GetAtomicNum()]= self.atomCount[atom.GetAtomicNum()]+1 if  atom.GetAtomicNum() in self.atomCount else 1
                if not (atom.GetAtomicNum()==6 and atom.IsInRing()):
                    candidates.append(FragmentExtensionByAtom(atom.GetAtomicNum(), atom.IsAromatic(), [atom.GetIdx()], [Embeding(molecule=mol)]))
        groups=[]
        candidates.sort(lambda a,b: cmp(self.atomCount[a.atomicNum], self.atomCount[b.atomicNum]))
        for extension in candidates:
            for ext in groups:
                if type(ext)==type(extension) and ext.IsEquivalent(extension):
                    ext.MergeFrom(extension)
                    break
            else:
                groups.append(extension)
        lst=self.atomCount.items()
        lst.sort(lambda a,b:cmp(a[1], b[1]))
        self.initialFragments=[]
        lst=[t[0] for t in lst]
        for extension in groups:
            if type(extension)==FragmentExtensionByAtom:
                f=Fragment(miner=self, excludeAtomList=lst[:lst.index(extension.atomicNum)])
            else:
                f=Fragment(miner=self)
            extension.Extend(f)
            self.initialFragments.append(f)
##        self.initialFragments.reverse()
##        excludeList=[]
##        for atom, c in lst:
##            f=Fragment(miner=self, excludeAtomList=excludeList)
##            f.InitializeFragment(atom)
##            self.initialFragments.append(f)
##            excludeList.append(atom)
        self.activeSet=set(self.active)
        self.inactiveSet=set(self.inactive)
            
    def TraverseTree(self, fragment):
        if self.canonicalPruning:
            codeWord=fragment.ToCannonicalSmiles()
            if codeWord in self.canonicalPruningSet:
                return self.canonicalPruningSet[codeWord]
            else:
                self.canonicalPruningSet[codeWord]=fragment.Support(self.activeSet)
        extended=fragment.Extend()
        extended=filter(lambda f:f.Support(self.activeSet)>=self.minSupport, extended)
        superStructSupport=[]
        for frag in extended:
            #print self.loader.WriteString(frag.ToOBMol())
            superStructSupport.append(self.TraverseTree(frag))
        support=fragment.Support(self.activeSet)
        if support>=self.minSupport and fragment.Support(self.inactiveSet)<=self.maxSupport:
            if not self.findClosed or (support not in superStructSupport):
                print fragment.ToSmiles().strip()+" %.2f %.2f" % (support, fragment.Support(self.inactiveSet))
                self.foundFragments.append(fragment)
        return support

    def Search(self):
        """Runs the search and returns the found fragments"""
        self.Initialize()
##        set_trace()
        self.foundFragments=[]
        for fragment in self.initialFragments:
            self.TraverseTree(fragment)
        #self.foundFragments=filter(lambda f:f.Support(self.inactive)<=self.maxSupport, self.foundFragments)
        return self.foundFragments

    def TraverseTreeIterator(self, fragment):
        if self.canonicalPruning:
            codeWord=fragment.ToCannonicalSmiles()
            if codeWord in self.canonicalPruningSet:
                raise StopIteration
            else:
                self.canonicalPruningSet[codeWord]=fragment.Support(self.activeSet)
        extended=fragment.Extend()
        extended=filter(lambda f:f.Support(self.activeSet)>=self.minSupport, extended)
        superStructSupport=[]
        for frag in extended:
            #print self.loader.WriteString(frag.ToOBMol())
            iter=self.TraverseTreeIterator(frag)
            try:
                while True:
                    f=iter.next()
                    superStructSupport.append(f.Support(self.active))
                    yield f
            except StopIteration:
                pass
                
            superStructSupport.append(self.TraverseTree(frag))
        support=fragment.Support(self.activeSet)
        if support>=self.minSupport and fragment.Support(self.inactiveSet)<=self.maxSupport:
            if not self.findClosed or (support not in superStructSupport):
                #print fragment.ToSmiles().strip()+" %.2f %.2f" % (support, fragment.Support(self.inactiveSet))
                #self.foundFragments.append(fragment)
                yield fragment
        #return support

    def SearchIterator(self):
        """Runs the search and returns the found fragments one by one"""
        self.Initialize()
##        set_trace()
        self.foundFragments=[]
        for fragment in self.initialFragments:
            iter=self.TraverseTreeIterator(fragment)
            try:
                while True:
                    yield iter.next()
            except StopIteration:
                pass
        #self.foundFragments=filter(lambda f:f.Support(self.inactive)<=self.maxSupport, self.foundFragments)
        #return self.foundFragments    
    
def LoadMolFromSmiles(smiles):
    """Returns an OBMol construcetd from an SMILES code"""
    smiles = sorted(smiles.split("."), key=len)[-1] ## Strip salts
    mol=OBMol()
    loader=OBConversion()
    loader.SetInAndOutFormats("smi","smi")
    if not loader.ReadString(mol, smiles):
        return None
    mol.smilesCode=smiles
    return mol
    
class Fragmenter(object):
    """An object that is used to fragment an ExampleTable
    Attributes:
        minSupport  : minimum frequency in the active set of the fragments to search for (default: 0.2)
        maxSupport  : maximum frequency in the inactive set of the fragments to search for (default: 0.2)
        findClosed  : finds only fragments that are not sub-structures of any other fragment with the same support (default: True)
    Example:
    >>> fragmenter=Fragmenter(minSupport=0.1, maxSupport=0.05)
    >>> data, fragments=fragmenter(data, "SMILES", lambda ex:ex.getclass())
    """
    def __init__(self, minSupport=0.2, maxSupport=0.2, canonicalPruning=True, findClosed=True):
        self.minSupport=minSupport
        self.maxSupport=maxSupport
        self.canonicalPruning=canonicalPruning
        self.findClosed=findClosed
    def __call__(self, data, smilesAttr=None, activeFunc=lambda e:True, useCannonicalFragments=True):
        """Takes a data-set, and runs the FragmentMiner on it. Returns a new data-set and the fragments.
        The new data-set contains new attributes that represent the presence of a fragment that was found.
        Arguments:
            data        : the dataset
            smilesAttr  : the attribute in the data that contains the SMILES codes
            activeFunc  : a function that takes an example from the data-set and returns True if the example should be
                    considered as active (if none is provided all examples are considered active)
            useCannonicalFragments  : if True use cannonical smiles for fragment attribute names else use regular smiles (defaut: True)
        """
        if not smilesAttr:
            smilesAttr=self.FindSmilesAttr(data)
        active=filter(lambda s:s, [str(e[smilesAttr]) for e in data if activeFunc(e)])
        inactive=filter(lambda s:s, [str(e[smilesAttr]) for e in data if not activeFunc(e)])
        
        miner=FragmentMiner(active, inactive, self.minSupport, self.maxSupport, canonicalPruning=self.canonicalPruning, findClosed=self.findClosed)
        self.fragments=fragments=miner.Search()
        fragVars=[orange.EnumVariable(frag.ToCannonicalSmiles() if useCannonicalFragments else frag.ToSmiles(), values=["0", "1"]) for frag in fragments]
        smilesInFragments=dict([(fragment, set([embeding.molecule.smilesCode for embeding in fragment.embedings]) ) for fragment in fragments])
        from functools import partial
        def getVal(var, fragment, smilesAttr, example, returnWhat):
            mol=LoadMolFromSmiles(str(example[smilesAttr]))
##            print "GetVal"
            return (fragment.ContainedIn(mol) and var(1) or var(0)) if mol else var(None)
        for var, frag in zip(fragVars, fragments):
            var.getValueFrom=partial(getVal,var, frag, smilesAttr)
        vars=data.domain.attributes+fragVars+(data.domain.classVar and [data.domain.classVar] or [])
        domain=orange.Domain(vars, data.domain.classVar and 1 or 0)
        domain.addmetas(data.domain.getmetas())
        table=orange.ExampleTable(domain)
        for e in data:
            vals=[e[attr] for attr in data.domain.attributes]+[1 if str(e[smilesAttr]) in smilesInFragments[fragment] else (None if e[smilesAttr].isSpecial() else 0) for fragment in fragments]
            vals=vals + [e.getclass()] if data.domain.classVar else vals
            ex=orange.Example(domain, vals)
            for key, val in e.getmetas().items():
                ex[key]=val
            table.append(ex)
        return table, fragments
    def FindSmilesAttr(self, data):
        data=data.select(orange.MakeRandomIndices2(data, min(20, len(data))))
        stringVars=filter(lambda var:type(var)==orange.StringVariable, data.domain.attributes+data.domain.getmetas().values())
        count=dict.fromkeys(stringVars, 0)
        for example in data:
            for var in stringVars:
                if LoadMolFromSmiles(str(example[var])):
                    count[var]+=1
        count=count.items()
        count.sort(lambda a,b:cmp(a[1], b[1]))
        return count[-1][0]
            
from Orange.orng import orngSVM
class FragmentBasedLearner(orange.Learner):
    """A learner wrapper class that first runs the molecular fragmentation on the data.
    Attributes:
        smilesAttr  : Attribute in the data that contains the smiles codes (if none is provided it will try to make a smart guess)
        learner     : learner that will be used to actualy learn on the fragmented data (default: orngSVM.SVMLearner)
        minSupport  : minimum frequency in the active set of the fragments to search for
        maxSupport  : maximum frequency in the inactive set of the fragments to search for
        activeFunc  : a function that takes an example from the learning data-set and returns True if the example should be
                    considered as active (if none is provided all examples are considered active)
        findClosed  : finds only fragments that are not sub-structures of any other fragment with the same support (default: True)
    """
    def __new__(cls, data=None, weights=0, **kwds):
        learner=orange.Learner.__new__(cls, **kwds)
        if data:
            learner.__init__(**kwds)
            return learner(data)
        else:
            return learner
    def __init__(self, learner=orngSVM.SVMLearner(probability=True), name="FragmentBasedLearner",
                 minSupport=0.2, maxSupport=0.2, smilesAttr=None, findClosed=True, activeFunc=lambda e:True):
        self.name=name
        self.learner=learner
        self.minSupport=minSupport
        self.smilesAttr=smilesAttr
        self.activeFunc=activeFunc
        self.maxSupport=maxSupport
        self.findClosed=findClosed
    def __call__(self, data, weight=0):
        fragmenter=Fragmenter(minSupport=self.minSupport, maxSupport=self.maxSupport, findClosed=self.findClosed)
        data, fragments=fragmenter(data, self.smilesAttr, self.activeFunc)
        return FragmentBasedClassifier(self.learner(data), data.domain)

class FragmentBasedClassifier(object):
    def __init__(self, classifier, domain):
        self.classifier=classifier
        self.domain=domain
    def __call__(self, example, getBoth=orange.GetValue):
        example=orange.Example(self.domain, example)
        return self.classifier(example, getBoth)

def Count(smiles, fragment):
    mols=filter(lambda m:m, map(LoadMolFromSmiles, smiles))
##    for mol in mols: mol.StripSalts()
    pattern=OBSmartsPattern()
    pattern.Init(fragment)
    return len(filter(lambda m:pattern.Match(m, True), mols))

def ContaindIn(smiles, fragment):
    mol=LoadMolFromSmiles(smiles)
    pattern=OBSmartsPattern()
    pattern.Init(fragment)
    return bool(pattern.Match(mol))


"""
Viasualizations
===============

"""
try:
    from oasa.svg_out import svg_out as _svg_out
    from oasa.pybel_bridge import PybelConverter
    from oasa.coords_generator import coords_generator
    from oasa import dom_extensions
except ImportError:
    try:
        from bkchem.oasa.oasa.svg_out import svg_out as _svg_out
        from bkchem.oasa.oasa.pybel_bridge import PybelConverter
        from bkchem.oasa.oasa.coords_generator import coords_generator
        from bkchem.oasa.oasa import dom_extensions
    except ImportError:
        class _svg_out(object):
            def __init__(self, *args, **kwargs):
                raise ImportError("OASA library not found")

try:
    from oasa.cairo_out import cairo_out as _cairo_out
except ImportError:
    try:
        from bkchem.oasa.oasa.cairo_out import cairo_out as _cairo_out
    except ImportError:
        class _cairo_out(object):
            def __init__(self, *args, **kwargs):
                raise ImportError("OASA library not found")

class cairo_out(_cairo_out):
    atom_colors = {"A":(0, 0, 0), "R":(255, 0, 0)}
    def __init__(self, **kwargs):
        _cairo_out.__init__(self, **kwargs)
        self.atom_colors.update({"A":(0, 0, 0), "R":(255, 0, 0)})

    def _draw_edge(self, e):
        v1, v2 = e.vertices
        s1, s2 = v1.symbol, v2.symbol
        matched = v1 in self.ob_matched and v2 in self.ob_matched
        ## 'A' and 'R' are used because they are in oasa periodic table and indicate any atom or any atom except hydrogen
        v1.symbol = "R" if matched else "A"
        v2.symbol = "R" if matched else "A"
        if matched:
            ## Set the default color argument for _draw_line to red
            self._draw_line.im_func.func_defaults = self._draw_line.im_func.func_defaults[:2]+ ((255, 0, 0),)
        _cairo_out._draw_edge(self, e)
        if matched:
            ## Restore the default color
            self._draw_line.im_func.func_defaults = self._draw_line.im_func.func_defaults[:2]+ ((0, 0, 0),)
        v1.symbol, v2.symbol = s1, s2

    def _draw_vertex(self, v):
        if v in self.ob_matched:
            color = self.atom_colors.get(v.symbol, None)
            self.atom_colors[v.symbol] = (255, 0, 0)
            _cairo_out._draw_vertex(self, v)
            ## Restore the atom colors
            if color:
                self.atom_colors[v.symbol] = color
            else:
                del self.atom_colors[v.symbol]
        else:
            _cairo_out._draw_vertex(self, v)

    def mol_to_cairo(self, molSmiles, fragSmiles=None, file="mol.png"):
        import pybel
        molSmiles = max(molSmiles.split("."), key = lambda s:len(s))
        mol = pybel.readstring("smi", molSmiles)
        
        if fragSmiles:
            pattern=pybel.Smarts(fragSmiles)
            matches = pattern.findall(mol)
            self.ob_matched = reduce(set.union, matches if matches != False else [], set())
        else:
            self.ob_matched = set()
#        from oasa.pybel_bridge import PybelConverter
        o_mol, idx2oa = PybelConverter.pybel_to_oasa_molecule_with_atom_map(mol)
#        from oasa.coords_generator import coords_generator
        gen = coords_generator(30)
        gen.calculate_coords(o_mol, bond_length=30, force=True)
##        self.oasa2obidx = dict([(value, key) for key, value in ix2oa.items()])
        self.ob_matched = [idx2oa[i] for i in self.ob_matched]
        self.color_bonds = True
        return _cairo_out.mol_to_cairo(self, o_mol, file)

import xml.dom.minidom as dom
class svg_out(_svg_out):
    def __init__(self, **kwargs):
        _svg_out.__init__(self, **kwargs)
        self._curr_color = "rgb(0%, 0%, 0%)"

    def _draw_edge(self, e):
        v1, v2 = e.vertices
        matched = v1 in self.ob_matched and v2 in self.ob_matched
        if matched:
            self._curr_color = "rgb(255, 0, 0)"
        else:
            self._curr_color = "rgb(0, 0, 0)"

        _svg_out._draw_edge(self, e)

    def _draw_vertex(self, v):
        if v in self.ob_matched:
            self._curr_color = "rgb(255, 0, 0)"
        else:
            self._curr_color = "rgb(0, 0, 0)"
        _svg_out._draw_vertex(self, v)


    def _draw_line( self, parent, start, end, line_width=1, capstyle=""):
        x1, y1 = start
        x2, y2 = end
#        from oasa import dom_extensions
        line = dom_extensions.elementUnder( parent, 'line',
                                            (( 'x1', str( x1)),
                                             ( 'y1', str( y1)),
                                             ( 'x2', str( x2)),
                                             ( 'y2', str( y2)),
                                             ( 'stroke', self._curr_color)))


    def _draw_text( self, parent, xy, text, font_name="Arial", font_size=16):
        x, y = xy
#        from oasa import dom_extensions
        dom_extensions.textOnlyElementUnder( parent, "text", text,
                                             (( "x", str( x)),
                                              ( "y", str( y)),
                                              ( "font-family", font_name),
                                              ( "font-size", str( font_size)),
                                              ( 'stroke', self._curr_color)))

    def mol_to_svg(self, molSmiles, fragSmiles=None):
        import pybel
        molSmiles = max(molSmiles.split("."), key = lambda s:len(s))
        mol = pybel.readstring("smi", molSmiles)
        
        if fragSmiles:
            pattern=pybel.Smarts(fragSmiles)
            matches = pattern.findall(mol)
            self.ob_matched = reduce(set.union, matches if matches != False else [], set())
        else:
            self.ob_matched = set()
#        from oasa.pybel_bridge import PybelConverter
        o_mol, idx2oa = PybelConverter.pybel_to_oasa_molecule_with_atom_map(mol)
#        from oasa.coords_generator import coords_generator
        gen = coords_generator(30)
        gen.calculate_coords(o_mol, bond_length=30, force=True)
##        self.oasa2obidx = dict([(value, key) for key, value in ix2oa.items()])
        self.ob_matched = [idx2oa[i] for i in self.ob_matched]
        return _svg_out.mol_to_svg(self, o_mol)    
        
def mol_to_png(molSmiles, fragSmiles=None, file="mol.png", size=None):
    d = cairo_out()
    d.mol_to_cairo(molSmiles, fragSmiles)
    if size != None:
        import Image
        image = Image.open(file)
        width, height = image.size
        resize_factor = float(size) / max(width, height)
        width, height = int(width * resize_factor), int(height * resize_factor)
        image = image.resize((width, height), Image.ANTIALIAS)
        newimage = Image.new("RGB", size=(size, size), color=(255, 255, 255))
        newimage.paste(image, (size - width, size - height))
        newimage.save(file)

def mol_to_svg(molSmiles, fragSmiles=None, file="mol.svg"):
    c = svg_out()
    tree = c.mol_to_svg(molSmiles, fragSmiles)
    if type(file) == str:
        file = open(file, "w")
    file.write( tree.toprettyxml())
    
def _test_draw():
##    mol_to_png("CC1CC2C(CC(=O)O2)OC3CC4C(CC(C5C(O4)CC=CCC6C(O5)CC=CC7C(O6)CCCC8C(O7)(CC9C(O8)CC2C(O9)C(CC(O2)CC(=C)C=O)O)C)C)OC3(C1)C", "Scc", "mol1.png")
##    mol_to_png("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "Scc", "mol1.png")
##    mol_to_png("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "NCC", "mol2.png")
##    mol_to_png("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "Ncc", "mol3.png")
##    mol_to_png("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "NCCC", "mol3.png")
    mol_to_svg("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "Scc", "mol1.svg")
    mol_to_svg("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "Ncc", "mol2.svg")
    mol_to_svg("CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "NCC", "mol3.svg")
    
def _test():
    import orange
    d=orange.ExampleTable("E:\chem\mutagen_raw.tab")
    active=[str(e["SMILES"]) for e in d if str(e[-1])=="1"]
    inactive=[str(e["SMILES"]) for e in d if str(e[-1])=="0"]
##    d=orange.ExampleTable("E:\PCLedit_s.tab")
##    active=[str(e["SMILES"]) for e in d if not e["SMILES"].isSpecial()][:100]
##    print active
##    inactive=[]
##    active=["NC(C)C(=O)O", "NC(CS)C(=O)O", "NC(CO)C(=O)O"]
##    active=["CCS(O)(O)N", "CCS(O)(C)N", "CS(O)(C)N", "CCS(=N)N", "CS(=N)N", "CS(=N)O"]
##    active=["NC(S)c1ccccc1","NCC1=CC=CC=C1", "NCC1C=CC=CC=1", "c1ccccc1C(N)C(=S(O)C)c2ccccc2"]
##    active=["c1ccccc1C(N)C(=S(O)C)c2ccccc2"]
##    active=["O=C1C=CC(=O)C=C1","O=C1CCCCCN1"]
##    active=["C1SC2CCN2C1C(=O)"]
##    active=["CCCCCCc1ccc(O)cc1O","Nc1ccc(O)c(N)c1", "Cc1cc(C)c(N)cc1C", "CN(C)C(=S)S[Zn]SC(=S)N(C)C", "NC(=O)N(CCO)N=O"]
##    active=["CC(C)CCCC(C)C1CCC2C3CC=C4CC(CCC4(C)C3CCC12C)OC(=O)Cc5ccc(cc5)N(CCCl)CCCl","Cc1cc(C)c(N=Nc2c(O)c(cc3cc(ccc23)S(=O)(=O)O)S(=O)(=O)O)cc1CCNNCc1ccc(cc1)C(=O)NC(C)C","CC(C)(Oc1ccc(cc1)C2CCCc3ccccc23)C(=O)O","CCn1cc(C(=O)O)c(=O)c2ccc(C)nc12"]
    active=["CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)Cl","CN1CCCCC1CCN2c3ccccc3Sc4c2cc(cc4)SC","c1ccc2c(c1)N(c3cc(ccc3S2)Cl)CCCN4CCN(CC4)CCO",
            "CN1CCN(CC1)CCCN2c3ccccc3Sc4c2cc(cc4)S(=O)(=O)N(C)C", "CN1CCC(=C2c3ccccc3Sc4c2cccc4)CC1", "[U]-C-S-P"]
##    active=["CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)Cl","CN1CCCCC1CCN2c3ccccc3Sc4c2cc(cc4)SC"]
##    active=["CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)C(F)(F)F", "CN(C)CCCN1c2ccccc2Sc3c1cc(cc3)Cl"]
##    set_trace()
    miner=FragmentMiner(active, inactive[:0], minSupport=0.1, maxSupport=0.1, addWholeRings=True, canonicalPruning=True)
    fragments=miner.Search()
##    for f in fragments:
##        print f.ToSmiles()

def _test1():
    import orange
    data=orange.ExampleTable("E:\chem\mutagen_raw.tab")
##    data=orange.ExampleTable("E:\chem\smiles.tab")
    fragmenter=Fragmenter(minSupport=0.02, maxSupport=0.1, canonicalPruning=True)
##    set_trace()
    data, fragments1=fragmenter(data, "SMILES") #, lambda e:str(e[-1])=="1")
##    data, fragments2=fragmenter(data, "SMILES", lambda e:str(e[-1])=="0")
    data.save("E:\chem\mutagen_raw_frag.tab")
    
if __name__=="__main__":
    import time
    sTime=time.clock()
    _test_draw()
    print time.clock()-sTime




"""
SOAP service client. Needs suds library.
 
"""
from __future__ import absolute_import

KEGG_WDSL = "http://soap.genome.jp/KEGG.wsdl"

import urllib2

def suds_service():
    """ Return an suds service object with kegg api service methods.
    
    >>> service = web_service()
    >>> service.list_databases()
    [(Definition){...

    """
    from suds.client import Client
    client = Client(KEGG_WDSL)
    return client.service

def suds_service_with_requests():
    import StringIO
    
    from suds.client import Client
    from suds.transport import Transport, TransportError
    from suds.properties import Unskin
    
    import requests

    class RequestsResponse(object):
        pass

    class RequestsTransport(Transport):
        def __init__(self, **kwargs):
            Transport.__init__(self)
            Unskin(self.options).update(kwargs)
            self.cookies = {}
            
        def send(self, request):
            result = None
            url = request.url
            message = request.message
            headers = request.headers
            # This does not seem to work, every request seems
            # to open a new connection
            headers["Connection"] = "Keep-Alive"
            try:
                response = requests.post(url, data=message, 
                                         headers=headers,
                                         cookies=self.cookies)
                    
                self.proxy = self.options.proxy
                response.raise_for_status()
                
                self.cookies.update(response.cookies)
                result = RequestsResponse()
                result.code = response.status_code
                result.headers = response.headers
                result.message = response.raw.read()
                return result
                
            except urllib2.HTTPError, e:
                if e.code in [202, 204]:
                    return None
                else:
                    raise TransportError(str(e), e.code, e.fp)
                
        def open(self, request):
            url = request.url
            message = request.message
            try:
                response = requests.get(url)
                self.proxy = self.options.proxy
                
                response.raise_for_status()
                return StringIO.StringIO(response.raw.read())
            except urllib2.HTTPError, e:
                raise TransportError(str(e), e.code, e.fp)
            
    transport = RequestsTransport()
    client = Client(KEGG_WDSL, transport=transport)
    return client.service
    
def SOAPy_service():
    import SOAPy

from . import conf

default_service = suds_service
web_service = suds_service

if conf.params["service.transport"] == "requests":
    try:
        import requests    
        web_service = suds_service_with_requests
    except ImportError:
        import warnings
        warnings.warn("requests package not installed.")
        
elif conf.params["service.transport"] == "urllib2":
    pass





"""
<name>Example Selector</name>
<description>Select examples based on input selectors.</description>
<icon>icons/SelectGenes.png</icon>
<priority>1070</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

from __future__ import absolute_import

from qttable import *

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .. import chipstat
from .OWDataFiles import DataFiles, ExampleSelection

class OWExampleSelector(OWWidget):
    settingsList  = ['negate', 'commitOnChange', 'sendNotSelectedData']

    def __init__(self, parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, 'Example Selector')
        self.callbackDeposit = []

        self.inputs = [("Example Selection", ExampleSelection, self.loadselection, Multiple + Default), ("Structured Data", DataFiles, self.chipdata)]
        self.outputs = [("Example Selection", ExampleSelection), ("Selected Structured Data", DataFiles, Default), ("Other Structured Data", DataFiles)]

        # Settings
        self.negate = 0
        self.commitOnChange = 1
        self.sendNotSelectedData = 1
        self.loadSettings()

        self.selectors = {}
        self.data = None

        # GUI
        # info
        box = QVGroupBox("Info", self.controlArea)
        self.infoa = QLabel('No data on input.', box)
        self.infob = QLabel('', box)
        OWGUI.separator(self.controlArea)
        box.setMinimumWidth(170)

        # gene selection
        self.layout=QVBoxLayout(self.mainArea)
        box = QVGroupBox("Gene Selection", self.mainArea)
        self.table=QTable(box)
        self.table.setSelectionMode(QTable.NoSelection)
        self.layout.add(box)
        self.table.hide()
        self.drawtable()

        OWGUI.checkBox(box, self, 'negate', 'Negate', callback = self.selectionChange)

        # output
        box = QVGroupBox("Output", self.controlArea)
        OWGUI.checkBox(box, self, 'sendNotSelectedData', 'Send not selected data', callback=self.selectionChange)
        OWGUI.checkBox(box, self, 'commitOnChange', 'Commit data on change')
        self.commitBtn = OWGUI.button(box, self, "Commit", callback=self.senddata, disabled=1)

        self.resize(700,100)
        
    def chipdata(self, data):
        if data:
            self.data = data
            if self.commitOnChange and len(self.selectors)>0 and len(self.data[0][1][0]) == len(self.selectors.values()[0][1]):
                self.senddata()
        else:
            self.send("Selected Structured Data", None)
            self.send("Other Structured Data", None)
            self.send("Example Selection", None)
        self.commitBtn.setEnabled(data <> None)

    def loadselection(self, selector, id):
        if selector:
            self.selectors[id] = list(selector + (1,0))    # the last two items for use and negation
        else:
            if id in self.selectors.keys(): del self.selectors[id]
        self.infoa.setText('%d selectors on input.' % len(self.selectors))
        self.drawtable()
##        print "debug OWExampleSelector.loadselection: self.commitOnChange: %s, len(self.selectors)>0: %s, self.data: %s, self.selectors.values(): %s" % (str(self.commitOnChange), str(len(self.selectors)>0), str(self.data), str(self.selectors.values()))
        if self.commitOnChange and len(self.selectors)>0 and self.data and len(self.data[0][1][0]) == len(self.selectors.values()[0][1]):
            self.senddata()
        self.commitBtn.setEnabled(self.selectors <> None and len(self.selectors))

    def drawtable(self):
        header = ['', 'Selector Name', 'Match #', 'Neg']
        self.table.setNumCols(len(header))
        self.header=self.table.horizontalHeader()
        for i in range(len(header)):
            self.header.setLabel(i, header[i])

        self.table.setNumRows(len(self.selectors))
        self.usesel_callback = [None] * len(self.selectors)
        self.tmpWidgets = []
        for (i, sel) in enumerate(self.selectors.values()):
            for (pos, k) in [(0,2), (3,3)]:
                cb = QCheckBox('', None)
                cb.setChecked(sel[k])
                self.usesel_callback[i] = lambda x, i=sel, k=k: self.usesel(x, i, k)
                self.connect(cb, SIGNAL("toggled(bool)"), self.usesel_callback[i])
                self.table.setCellWidget(i, pos, cb)
                self.tmpWidgets.append(cb)
            
            self.table.setText(i, 1, sel[0])
            self.table.setText(i, 2, '%d (%4.1f%s)' % (sel[1].count(1), 100.*sel[1].count(1)/len(sel[1]), '%') )
        self.table.setLeftMargin(0)
        self.table.verticalHeader().hide()

        for i in range(len(header)):
            self.table.adjustColumn(i)

        self.table.show()

    def usesel(self, x, sel, k):
        sel[k] = int(x)
        if self.commitOnChange:
            self.senddata()

    def senddata(self):
        if len(self.selectors):
            self.progressBarInit()
            sel = []
            for s in self.selectors.values():
                if s[2]: # used?
                    if s[3]: # negated?
                        sel.append([not x for x in s[1]])
                    else:
                        sel.append(s[1])
            if len(sel) == 0:
                match = [0]*len(self.selectors.values()[0][1])
            elif len(sel)>1:
##                match = apply(map, (max, ) + tuple(sel))
                match = apply(map, (min, ) + tuple(sel))
            else:
                match = sel[0]
            if self.negate:
                match = [not x for x in match]

            nmatch = match.count(1)
            self.infob.setText("%d genes (%4.1f%s) match criteria" % (nmatch, 100.*nmatch/len(self.selectors.values()[0][1]), '%'))

            self.match = match
##            self.progressBarSet(20)

            self.send("Example Selection", self.match)
            if self.data:
                n = self.match.count(1)
                if n==len(self.data[0][1][0]): # all genes match
                    self.progressBarFinished()
                    self.send("Selected Structured Data", self.data)
                    self.send("Other Structured Data", None)
                elif n==0:
                    self.progressBarFinished()
                    self.send("Selected Structured Data", None)
                    if self.sendNotSelectedData:
                        self.send("Other Structured Data", self.data)
                    else:
                        self.send("Other Structured Data", None)
                else:
                    try:
                        P = []; N = []
                        if self.sendNotSelectedData:
                            pbStep = 50. / len(self.data)
                        else:
                            pbStep = 100. / len(self.data)
                        for (strainname, data) in self.data:
                            dataP = [d.select(self.match) for d in data]
                            for i in range(len(data)):
                                dataP[i].name = data[i].name
                            P.append((strainname, dataP))
                            self.progressBarAdvance(pbStep)
                        self.send("Selected Structured Data", P)
                        if self.sendNotSelectedData:
                            for (strainname, data) in self.data:
                                dataN = [d.select(self.match, negate=1) for d in data]
                                for i in range(len(data)):
                                    dataN[i].name = data[i].name
                                N.append((strainname, dataN))
                                self.progressBarAdvance(pbStep)
                            self.send("Other Structured Data", N)
                        else:
                            self.send("Other Structured Data", None)
                    except:
                        print "debug OWExampleSelector.senddata\n\tlen(d): %i\n\tlen(self.match): %i\n\t" % (len(d), len(self.match))
                        raise
            self.progressBarFinished()

    def selectionChange(self):
        if self.commitOnChange:
            self.senddata()


if __name__=="__main__":
    a=QApplication(sys.argv)
    ow=OWExampleSelector()
    a.setMainWidget(ow)
    ow.show()
    a.exec_loop()
    ow.saveSettings()



"""
<name>MeSH Browser</name>
<description>Browse MeSH ontology.</description>
<icon>icons/MeSH.png</icon>
<contact>Crt Gorup (crt.gorup@gmail.com)</contact> 
<priority>2040</priority>
"""

from __future__ import absolute_import

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from ..obiMeSH import *

class MyQTableWidgetItem(QTableWidgetItem):
	""" Our implementation of QTable item allowing numerical sorting.  """
	def __init__(self,table,text):
		QTableWidgetItem.__init__(self,table,QTableWidgetItem.Never,text)
		self.data = text
		
	def key(self):	  # additional setting to correctly handle text and numerical sorting
		try:	# sorting numerical column
			e = float(self.data)
			tdata = pval = "%.4g" % e
			l = len(self.data)
			pre = ""
			
			offset = 0	# additional parameter to hadle exponent '2e-14' float format
			if(tdata.count('e')>0 or tdata.count('E')>0):
				pre="*"
				offset = 1
				
			for i in range(0,40-l-offset):
				pre = pre + "0"
			return pre + tdata
		except ValueError:	  # sorting text column
			return self.data 

class ListViewToolTip(QToolTip):
	"""brief A class to allow tooltips in a listview."""
	def __init__(self, view, column, data):
		"""brief ListViewToolTip constructor.
		\param view		  QListView instance
		\param column		Listview column
		\param truncatedOnly Only display the tooltip if the column data is truncated
		"""
		QToolTip.__init__(self, view.viewport())
		self.__view = view
		self.__col  = column
		self.__data = data	  # mapping from name -> description
		# self.setWakeUpDelay(400)

	def appendData(self,data):
		self.__data = data

	def maybeTip(self, pos):
		"""brief Draw the tooltip.
		\param pos Tooltip position.
		"""
		item = self.__view.itemAt(pos)
		if item is not None:
			if(self.__data.has_key(str(item.text(self.__col)))):
				tipString = self.__data[str(item.text(self.__col))]
				counter = 45
				newTipString = ""
				for i in tipString.split(" "):
					if counter < 0:
						newTipString = newTipString + i + "\n"
						counter = 45
					else:
						newTipString = newTipString + i + " "
						counter -= len(i)
				tipString = newTipString
				cr = self.__view.itemRect(item)
				headerPos = self.__view.header().sectionPos(self.__col)
				cr.setLeft(headerPos)
				cr.setRight(headerPos + self.__view.header().sectionSize(self.__col))
				self.tip(cr, tipString)
			else:
				print item.text(self.__col) + " not found in toDecs"

class OWMeSHBrowser(OWWidget):
	settingsList = ["multi", "maxPValue", "minExamplesInTerm"]
	def __init__(self,parent=None,signalManager=None):
		OWWidget.__init__(self,parent,signalManager,"MeshBrowser")
		self.inputs = [("Reference data", ExampleTable, self.getReferenceData),("Cluster data", ExampleTable, self.getClusterData) ]
		self.outputs = [("Selected examples", ExampleTable)]

		# widget variables
		self.loadedRef = 0
		self.loadedClu = 0
		self.maxPValue = 0.05
		self.minExamplesInTerm = 5
		self.multi = 1
		self.reference = None
		self.cluster = None
		self.loadSettings()		
		self.mesh = obiMeSH() # main object is created
		self.dataLoaded = self.mesh.dataLoaded

		# left pane
		box = OWGUI.widgetBox(self.controlArea, "Info")
		#box = QGroupBox("Info", self.controlArea)
		self.infoa = OWGUI.label(box, self, "No reference data.")
		self.infob = OWGUI.label(box, self, "No cluster data.")
		self.ratio = OWGUI.label(box, self, "")
		self.ref_att = OWGUI.label(box, self, "")
		self.clu_att = OWGUI.label(box, self, "")
		self.resize(960,600)
		OWGUI.separator(self.controlArea)
		
		self.optionsBox = OWGUI.widgetBox(self.controlArea, "Options")
		self.maxp = OWGUI.lineEdit(self.optionsBox, self, "maxPValue", label="threshold:", orientation="horizontal", labelWidth=120, valueType=float)
		self.minf = OWGUI.lineEdit(self.optionsBox, self, "minExamplesInTerm", label="min. frequency:", orientation="horizontal", labelWidth=120, valueType=int)		
		#OWGUI.checkBox(self.optionsBox, self, 'multi', 'Multiple selection', callback= self.checkClicked)
		OWGUI.button(self.optionsBox, self, "Refresh", callback=self.refresh)

		# right pane
		self.col_size = [280,84,84,100,110]
		self.sort_col = 0
		self.sort_dir = True
		self.columns = ['MeSH term', '# reference', '# cluster', 'p value','fold enrichment'] # both datasets

		self.splitter = QSplitter(Qt.Vertical, self.mainArea)
		self.mainArea.layout().addWidget(self.splitter)
		   
		# list view
		self.meshLV = QTreeWidget(self.splitter)
		#self.meshLV.setSelectionMode(QAbstractItemView.MultiSelection)
		self.meshLV.setAllColumnsShowFocus(1)
		self.meshLV.setColumnCount(len(self.columns))
		self.meshLV.setHeaderLabels(self.columns)
		
		self.meshLV.header().setClickable(True)
		#self.meshLV.header().setSortIndicatorShown(True)
		#self.meshLV.setSortingEnabled(True)
		self.meshLV.setRootIsDecorated (True)
		self.connect(self.meshLV, SIGNAL("itemSelectionChanged()"), self.viewSelectionChanged)
		#self.meshLV.setItemDelegateForColumn(3, EnrichmentColumnItemDelegate(self))
		#self.tooltips = ListViewToolTip(self.meshLV,0, self.mesh.toDesc)		

		# table of significant mesh terms
		self.sigTermsTable = QTableWidget(self.splitter)
		self.sigTermsTable.setColumnCount(len(self.columns))
		self.sigTermsTable.setRowCount(4)
		## hide the vertical header
		self.sigTermsTable.verticalHeader().hide()
		#self.sigTermsTable.setLeftMargin(0)
		#self.sigTermsTable.setSelectionMode(QAbstractItemView.MultiSelection)
		
		for i in range(0,len(self.columns)):
			self.sigTermsTable.horizontalHeader().resizeSection(i,self.col_size[i])
			self.meshLV.header().resizeSection(i,self.col_size[i])

		self.sigTermsTable.setHorizontalHeaderLabels(self.columns)

		self.connect(self.sigTermsTable, SIGNAL("itemSelectionChanged()"), self.tableSelectionChanged) 
		self.connect(self.sigTermsTable, SIGNAL("clicked(int,int,int,const QPoint&)"), self.tableClicked)
		self.splitter.show()
		self.optionsBox.setDisabled(1)

	def tableSelectionChanged(self):
		return True

	def tableClicked(self,row,col,button, point):
		if self.sort_col == col:
			self.sort_dir = not self.sort_dir
		else:
			self.sort_col = col
			self.sort_dir = True
		
		self.sigTermsTable.sortItems(self.sort_col,Qt.DescendingOrder)
		#print "sortiram ", col, " ",row

	def checkClicked(self):
		if self.multi == 0:
			self.meshLV.clearSelection()
		self.meshLV.setSelectionMode(QAbstractItemView.MultiSelection)

	def viewSelectionChanged(self):
		"""
		Function viewSelectionChanged is used to handle the widget output once the user clicks or selects MeSH term inside the tree view.
		"""
		items = list()
		self.progressBarInit()

		itms = self.meshLV.selectedItems()
		for i in itms:
			items.append(i.term)
		
		#for i in self.lvItem2Mesh.iterkeys():
		#	if i.isSelected():
		#		items.append(self.lvItem2Mesh[i][1])
		#print "selecting ", items
		
		if self.reference:
			data=self.mesh.findSubset(self.reference, items, callback=self.progressBarSet, MeSHtype='term')
		else:
			data=self.mesh.findSubset(self.cluster, items, callback=self.progressBarSet, MeSHtype='term')
		
		#print items
		self.send("Selected examples", data)
		self.progressBarFinished()

	def __updateData__(self):
		"""
		Function __updateData__ is used to display the results of the MeSH term enrichment analysis inside the widget components.
		"""
		self.lvItem2Mesh = dict()

		if(self.reference and self.cluster):
			if(len(self.cluster) > len(self.reference)):
				self.optionsBox.setDisabled(1)
				return False
			# everything is ok, now we can calculate enrichment and update labels, tree view and table data
			#self.warning()
			self.optionsBox.setDisabled(0)
			self.progressBarInit()
			self.treeInfo, self.results = self.mesh.findEnrichedTerms(self.reference,self.cluster,self.maxPValue, treeData= True, callback= self.progressBarSet)
			self.progressBarFinished()
			self.ratio.setText("ratio = %.4g" % self.mesh.ratio)
			self.clu_att.setText("cluster MeSH att: " + self.mesh.clu_att)
			self.ref_att.setText("reference MeSH att: " + self.mesh.ref_att)

			# table data update
			self.sigTermsTable.setRowCount(len(self.results))
			index = 0
			for i in self.results.iterkeys(): ## sorted by the p value
			   # mTerm = i[0]
				mID = self.mesh.toName[i] + " (" + i + ")"
				rF = self.results[i][0]
				cF = self.results[i][1]
				pval = self.results[i][2]
				fold = self.results[i][3]
				pval = "%.4g" % pval
				fold = "%.4g" % fold
				vals = [mID ,rF,cF, pval, fold]
				for j in range(len(vals)):
					self.sigTermsTable.setItem(index,j, QTableWidgetItem(str(vals[j])))
				index = index + 1

			# initial sorting - p value
			self.sort_col = 3
			self.sort_dir = True
			self.sigTermsTable.sortItems(self.sort_col, Qt.DescendingOrder)
		
			# tree view update - The most beautiful part of this widget!
			starters = self.treeInfo["tops"]	   # we get a list of possible top nodes			
			self.meshLV.clear()

			for e in starters:	  # we manualy create top nodes
				f = QTreeWidgetItem(self.meshLV);
				#f.setOpen(1)
				rfr = str(self.results[e][0])
				cfr = str(self.results[e][1])
				pval = "%.4g" % self.results[e][2]
				fold = "%.4g" % self.results[e][3]
				self.lvItem2Mesh[f] = (self.mesh.toName[e],e)
				data = [self.mesh.toName[e], rfr, cfr, pval, fold]
				f.term = self.mesh.toName[e]
				for t in range(len(data)):
					f.setText(t,data[t])
				self.__treeViewMaker__(f, e, False)
			self.meshLV.expandAll()

		elif self.reference or self.cluster:
			if self.reference:
				current_data = self.reference
			else:
				current_data = self.cluster

			self.optionsBox.setDisabled(0)
			self.progressBarInit()
			self.treeInfo, self.results=self.mesh.findFrequentTerms(current_data,self.minExamplesInTerm, treeData=True, callback=self.progressBarSet)
 			self.progressBarFinished()
			if self.reference:
				self.ref_att.setText("reference MeSH att: " + self.mesh.solo_att)
			else:
				self.clu_att.setText("cluster MeSH att: " + self.mesh.solo_att)

			# table data update
			self.sigTermsTable.setRowCount(len(self.results))
			index = 0
			for i in self.results.iterkeys(): 
				mID = self.mesh.toName[i] + " (" + i + ")"
				rF = self.results[i]
				vals = [mID ,rF]
				for j in range(len(vals)):
					self.sigTermsTable.setItem(index,j, QTableWidgetItem(str(vals[j])))
				index = index + 1

			# initial sorting - frequency
			self.sort_col = 1
			self.sort_dir = True
			self.sigTermsTable.sortItems(self.sort_col, Qt.DescendingOrder)

			# tree view update - The most beautiful part of this widget!
			starters = self.treeInfo["tops"]	   # we get a list of possible top nodes
			self.meshLV.clear()
			
			for e in starters:	  # we manualy create top nodes
				f = QTreeWidgetItem(self.meshLV);
				#f.setOpen(1)
				self.lvItem2Mesh[f] = (self.mesh.toName[e],e)
				rfr = str(self.results[e])
				data = [self.mesh.toName[e], rfr]
				f.term = self.mesh.toName[e]
				for t in range(len(data)):
					f.setText(t,data[t])
				self.__treeViewMaker__(f, e, True)
			self.meshLV.expandAll()
			
	def __treeViewMaker__(self,parentLVI, parentID, soloMode):
		"""
		Function __treeViewMaker__ is used to build the tree in treeListView. When soloMode=True function only displays \
		first two columns inside the tree view (suitable when only one dataset is present).
		"""
		for i in self.treeInfo[parentID]:   # for each succesor
			f = QTreeWidgetItem(parentLVI);
			f.term = self.mesh.toName[i]
			data = [self.mesh.toName[i]]
			if soloMode:
				rfr = str(self.results[i])
				data.append(rfr)
			else:		   # when we have referece and cluster dataset we have to print additional info
				rfr = str(self.results[i][0])
				cfr = str(self.results[i][1])
				pval = "%.4g" % self.results[i][2]
				fold = "%.4g" % self.results[i][3]
				data.extend([rfr, cfr, pval, fold])
			self.lvItem2Mesh[f]=(data[0],i)		# mapping   QTreeWidgetItem <-> mesh id
			for t in range(len(data)):
				f.setText(t,data[t])
			self.__treeViewMaker__(f,i, soloMode)			
		return True

	def refresh(self):
		"""
		Function refresh is executed as number of widget inputs changes.
		"""
		if self.reference or self.cluster:		
			self.__switchGUI__() 
			self.__updateData__()

	def __clearGUI__(self):
		"""
		Function __clearGUI__ sets tree view, table and labels to their default values.
		"""
		self.meshLV.clear()
		self.sigTermsTable.setRowCount(0)

	def __switchGUI__(self):
		"""
		Function __switchGUI__ is capable of changing GUI based on number of connected inputs.
		"""
		if(len(self.cluster) > len(self.reference)):
			self.optionsBox.setDisabled(1)
			#self.warning("Cluster dataset is greater than reference dataset. Please check the widget inputs.")
			QMessageBox.warning( None, "Invalid input dataset length", "Cluster dataset is longer than the reference dataset. Please check the widget inputs." , QMessageBox.Ok)
			return False
		if not self.reference and not self.cluster:
			self.optionsBox.setDisabled(1)
			return
		self.optionsBox.setDisabled(0)
		solo = True
		if self.reference and self.cluster:
			solo = False
		if solo:
			self.maxp.setDisabled(1)
			self.minf.setDisabled(0)
			for i in range(2,len(self.columns)):
				self.meshLV.hideColumn(i)
				self.sigTermsTable.hideColumn(i)
			
			self.sigTermsTable.setHorizontalHeaderLabels(["MeSH term","frequency"])
			self.meshLV.setHeaderLabels(["MeSH term","frequency"])
			self.ratio.setText("")
		else:
			self.maxp.setDisabled(0)
			self.minf.setDisabled(1)
			for i in range(0,len(self.columns)):
				self.meshLV.showColumn(i)
				self.sigTermsTable.showColumn(i)
			self.sigTermsTable.setHorizontalHeaderLabels(self.columns)
			self.meshLV.setHeaderLabels(self.columns)
			for i in range(0,len(self.columns)):
				self.meshLV.header().resizeSection(i,self.col_size[i])
				self.sigTermsTable.horizontalHeader().resizeSection(i,self.col_size[i])
			self.ratio.setText("ratio = %.4g" % self.mesh.ratio)

	def getReferenceData(self, data):
		"""
		Function getReferenceData is executed once reference signal is connected to the widget.
		"""
		if data:
			self.reference = data
			self.infoa.setText('%d reference examples' % len(data))
		else:
			self.reference = None
			self.infoa.setText('No reference data.')
			self.ref_att.setText('')

		if self.reference or self.cluster:		
			self.__switchGUI__() 
			self.__updateData__()
		else:
			self.__clearGUI__()

	def getClusterData(self, data):
		"""
		Function getClusterData is executed once cluster signal is connected to the widget.
		"""
		if data:
			self.cluster = data
			self.infob.setText('%d cluster examples' % len(data))
		else:
			self.cluster = None
			self.infob.setText('No cluster data.')
			self.clu_att.setText('')

		if self.reference or self.cluster:		
			self.__switchGUI__() 
			self.__updateData__()
		else:
			self.__clearGUI__()



"""<name>MA Plot</name>
<description>Normalize expression array data on a MA - plot</description>
<icon>icons/Normalize.png</icons>
"""

from __future__ import absolute_import

from functools import partial

import numpy
        
from Orange.OrangeWidgets import OWConcurrent
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *
from Orange.OrangeWidgets.OWGraph import *

from .. import obiExpression

class ProgressBarDiscard(QObject):
    def __init__(self, parent, redirect):
        QObject.__init__(self, parent)
        self.redirect = redirect
        self._delay = False
        
    @pyqtSignature("progressBarSet(float)")
    def progressBarSet(self, value):
        # Discard OWBaseWidget.progressBarSet call, because it calls qApp.processEvents
        #which can result in 'event queue climbing' and max. recursion error if GUI thread
        #gets another advance signal before it finishes with this one
        if not self._delay:
            try:
                self._delay = True
                self.redirect.progressBarSet(value)
            finally:
                self._delay = False
        
        
class OWMAPlot(OWWidget):
    settingsList = ["appendZScore", "appendRIValues"]
    contextHandlers = {"": DomainContextHandler("", ["selectedGroup", "zCutoff"])}
    
    CENTER_METHODS = [("Average", obiExpression.MA_center_average),
                      ("Lowess (fast - interpolated)", obiExpression.MA_center_lowess_fast),
                      ("Lowess", obiExpression.MA_center_lowess)]
    
    MERGE_METHODS = [("Average", numpy.ma.average),
                     ("Median", numpy.ma.median),
                     ("Geometric mean", obiExpression.geometric_mean)]
    
    def __init__(self, parent=None, signalManager=None, name="Normalize Expression Array"):
        OWWidget.__init__(self, parent, signalManager, name, wantGraph=True)
        
        self.inputs = [("Expression array", ExampleTable, self.setData)]
        self.outputs = [("Normalized expression array", ExampleTable, Default), ("Filtered expression array", ExampleTable)]
        
        self.selectedGroup = 0
        self.selectedCenterMethod = 0
        self.selectedMergeMethod = 0
        self.zCutoff = 1.96
        self.appendZScore = False
        self.appendRIValues = False
        self.autoCommit = False
        
        self.loadSettings()
        ## GUI
        self.infoBox = OWGUI.widgetLabel(OWGUI.widgetBox(self.controlArea, "Info", addSpace=True),
                                         "No data on input.")
        
        box = OWGUI.widgetBox(self.controlArea, "Split by", addSpace=True)
        self.groupCombo = OWGUI.comboBox(box, self, "selectedGroup", 
                                         callback=self.onGroupSelection
                                         )
        
        self.centerCombo = OWGUI.comboBox(self.controlArea, self, "selectedCenterMethod",
                                          box="Center Fold-change Using",
                                          items=[name for name, _ in self.CENTER_METHODS],
                                          callback=self.onCenterMethodChange,
                                          addSpace=True
                                          )
        
        self.mergeCombo = OWGUI.comboBox(self.controlArea, self, "selectedMergeMethod",
                                         box="Merge Replicates",
                                         items=[name for name, _ in self.MERGE_METHODS],
                                         tooltip="Select the method for replicate merging",
                                         callback=self.onMergeMethodChange,
                                         addSpace=True
                                         )
        
        box = OWGUI.doubleSpin(self.controlArea, self, "zCutoff", 0.0, 3.0, 0.01,
                               box="Z-Score Cutoff",
                               callback=[self.replotMA, self.commitIf])
        
        OWGUI.separator(self.controlArea)
        
        box = OWGUI.widgetBox(self.controlArea, "Ouput")
        OWGUI.checkBox(box, self, "appendZScore", "Append Z-Scores",
                       tooltip="Append calculated Z-Scores to output",
                       callback=self.commitIf
                       )
        
        OWGUI.checkBox(box, self, "appendRIValues", "Append Log Ratio and Intensity values",
                       tooltip="Append calculated Log Ratio and Intensity values to output data",
                       callback=self.commitIf
                       )
        
        cb = OWGUI.checkBox(box, self, "autoCommit", "Commit on change",
                       tooltip="Commit data on any change",
                       callback=self.commitIf
                       )
        
        b = OWGUI.button(box, self, "Commit", callback=self.commit)
        OWGUI.setStopper(self, b, cb, "changedFlag", callback=self.commit)
        
        self.connect(self.graphButton, SIGNAL("clicked()"), self.saveGraph)
        
        OWGUI.rubber(self.controlArea)
        self.graph = OWGraph(self.mainArea)
        self.graph.setAxisTitle(QwtPlot.xBottom, "Intensity: log<sub>10</sub>(R*G)")
        self.graph.setAxisTitle(QwtPlot.yLeft, "Log ratio: log<sub>2</sub>(R/G)")
        self.graph.showFilledSymbols = True
        self.mainArea.layout().addWidget(self.graph)
        self.groups = []
        self.split_data = None, None
        self.merged_splits = None, None
        self.centered = None, None
        self.changedFlag = False
        self.data = None
        
        self.resize(800, 600)
        
    def onFinished(self, status):
        self.setEnabled(True)
    
    def onUnhandledException(self, ex_info):
        self.setEnabled(True)
        print >> sys.stderr, "Unhandled exception in non GUI thread"
        
        ex_type, ex_val, tb = ex_info
        if ex_type == numpy.linalg.LinAlgError and False:
            self.error(0, "Linear algebra error: %s" % repr(ex_val))
        else:
            sys.excepthook(*ex_info)
    
    def onGroupSelection(self):
        if self.data:
            self.updateInfoBox()
            self.splitData()
            self.runNormalization()
        
    def onCenterMethodChange(self):
        if self.data:
            self.runNormalization()
        
    def onMergeMethodChange(self):
        if self.data:
            self.splitData()
            self.runNormalization()
        
    def proposeGroups(self, data):
        col_labels = [attr.attributes.items() for attr in data.domain.attributes]
        col_labels = sorted(reduce(set.union, col_labels, set()))
        col_labels = [(key, value, 1) for key, value in col_labels]
        
        attrs = [attr for attr in data.domain.variables + data.domain.getmetas().values() \
                 if attr.varType == orange.VarTypes.Discrete]
        
        row_labels = [(attr.name, value, 0) for attr in attrs for value in attr.values]
        
        def filterSingleValues(labels):
            ret = []
            for name, value, axis in labels:
                match = [(n, v, a) for n, v, a in labels if n == name]
                if len(match) > 1:
                    ret.append((name, value, axis))
            return ret
            
        col_labels = filterSingleValues(col_labels)
        row_labels = filterSingleValues(row_labels)
        
        return col_labels + row_labels
    
    def setData(self, data):
        self.closeContext("")
        self.data = data
        self.error([0 ,1])
        if data is not None:
            self.infoBox.setText("%i genes on input" % len(data))
            self.groups = self.proposeGroups(data)
            self.groupCombo.clear()
            self.groupCombo.addItems(["%s: %s" % (key, value) for key, value, axis in self.groups])
            
            if not self.groups:
                self.error(1, "Input data has no class attribute or attribute labels!")
                self.clear()
                return
            
            self.openContext("", data)
            self.selectedGroup = min(self.selectedGroup, len(self.groups) - 1)
            
            self.updateInfoBox()
            self.splitData()
            self.runNormalization()
        else:
            self.clear()
        
    def clear(self):
        self.groups = []
        self.data = None
        self.centered = None, None
        self.split_data = None, None
        self.merged_splits = None, None
        self.graph.removeDrawingCurves()
        self.infoBox.setText("No data on input")
        self.send("Normalized expression array", None)
        self.send("Filtered expression array", None)
        
    def updateInfoBox(self):
        genes = self.getGeneNames()
        self.infoBox.setText("%i genes on input" % len(self.data))
        
    def getSelectedGroup(self):
        return self.groups[self.selectedGroup]
    
    def getSelectedGroupSplit(self):
        key, value, axis = self.getSelectedGroup()
        other_values = [v for k, v, a in self.groups if k == key and a == axis and v != value]
        return [(key, value), (key, other_values)], axis
    
    def getGeneNames(self):
        key, value, axis = self.getSelectedGroup()
        if axis == 0:
            genes = [str(ex[key]) for ex in self.data]
        else:
            genes = [attr.name for attr in self.data.domain.attributes]
            
        return genes
    
    def splitData(self): 
        groups, axis = self.getSelectedGroupSplit()
        self.split_ind = [obiExpression.select_indices(self.data, key, value, axis) for key, value in groups]
        self.split_data = obiExpression.split_data(self.data, groups, axis)
        
    def getMerged(self):
        split1, split2 = self.split_data
        (array1, _, _), (array2, _, _) = split1.toNumpyMA(), split2.toNumpyMA()
        
        _, _, axis = self.getSelectedGroup()
        merge_function = self.MERGE_METHODS[self.selectedMergeMethod][1]
        
        merged1 = obiExpression.merge_replicates(array1, axis, merge_function=merge_function)
        merged2 = obiExpression.merge_replicates(array2, axis, merge_function=merge_function)
        self.merged_splits = merged1, merged2
        
        return self.merged_splits
        
    def runNormalization(self):
        self.progressBarInit()
        self.progressBarSet(0.0)
        G, R = self.getMerged()
        self.progressBarSet(5.0)
        
        center_method = self.CENTER_METHODS[self.selectedCenterMethod][1]
        
        # TODO: progess bar , lowess can take a long time
        if self.selectedCenterMethod in [1, 2]: #Lowess
            Gc, Rc = center_method(G, R, f = 1./min(500., len(G)/100), iter=1)
        else:
            Gc, Rc = center_method(G, R)
        self.progressBarSet(70.0)
        self.centered = Gc, Rc
        self.z_scores = obiExpression.MA_zscore(Gc, Rc, 1./3.)
        self.progressBarSet(100.0)
        self.plotMA(Gc, Rc, self.z_scores, self.zCutoff)
        self.progressBarFinished()
        
    def runNormalizationAsync(self):
        """ Run MA centering and z_score estimation in a separate thread 
        """
        self.error(0)
        self.progressBarInit()
        self.progressBarSet(0.0)
        G, R = self.getMerged()
#        self.progressBarSet(5.0)
        
        center_method = self.CENTER_METHODS[self.selectedCenterMethod][1]
        use_lowess = self.selectedCenterMethod in [1, 2]
        
        def run(progressCallback = lambda value: None): # the function to run in a thread
#            progressCallback(5.0)
            if use_lowess:
                Gc, Rc = center_method(G, R, f=2./3., iter=1, progressCallback=lambda val: progressCallback(val/2))
            else:
                Gc, Rc = center_method(G, R)
            progressCallback(50)
            z_scores = obiExpression.MA_zscore(Gc, Rc, 1./3., progressCallback= lambda val: progressCallback(50 + val/2))
            
            return Gc, Rc, z_scores
        
        self.progressDiscard = ProgressBarDiscard(self, self)
         
        async = self.asyncCall(run, name="Normalization",
                               onResult=self.onResults,
                               onError=self.onUnhandledException)
        self.connect(async, SIGNAL("progressChanged(float)"), self.progressDiscard.progressBarSet, Qt.QueuedConnection)
        self.setEnabled(False)
        async.__call__(progressCallback=async.emitProgressChanged)
            
    ## comment out this line if threading creates any problems 
    runNormalization = runNormalizationAsync
    
    def onResults(self, (Gc, Rc, z_scores)):
        """ Handle the results of centering and z-scoring
        """
        assert(QThread.currentThread() is self.thread())
        self.setEnabled(True)
        qApp.processEvents()
        self.progressBarFinished()
        self.centered = Gc, Rc
        self.z_scores = z_scores
        self.plotMA(Gc, Rc, z_scores, self.zCutoff)
        self.commit()
        
    
    def plotMA(self, G, R, z_scores, z_cuttof):
        ratio, intensity = obiExpression.ratio_intensity(G, R)
        
        filter = numpy.isfinite(ratio) & numpy.isfinite(intensity) & numpy.isfinite(z_scores)
        for array in [ratio, intensity, z_scores]:
            if numpy.ma.is_masked(array):
                filter &= -array.mask
        
        filtered_ind = numpy.ma.where(filter)
        ratio = numpy.take(ratio, filtered_ind)
        intensity = numpy.take(intensity, filtered_ind)
        z_scores = numpy.take(z_scores, filtered_ind)
        
        red_ind = numpy.ma.where(numpy.ma.abs(z_scores) >= z_cuttof)
        blue_ind = numpy.ma.where(numpy.ma.abs(z_scores) < z_cuttof)
        
        red_xdata, red_ydata = intensity[red_ind], ratio[red_ind]
        blue_xdata, blue_ydata = intensity[blue_ind], ratio[blue_ind]
        self.graph.removeDrawingCurves()
        
        c = self.graph.addCurve("", Qt.black, Qt.black, xData=[0.0, 1.0], yData=[0.0, 0.0], style=QwtPlotCurve.Lines, symbol=QwtSymbol.NoSymbol)
        c.setAxis(QwtPlot.xTop, QwtPlot.yLeft)
        
        self.graph.addCurve("Z >= %.2f" % z_cuttof, QColor(Qt.red), Qt.red, brushAlpha=100, size=6, enableLegend=True, xData=list(red_xdata), yData=list(red_ydata), autoScale=True)
        self.graph.addCurve("Z < %.2f" % z_cuttof, QColor(Qt.blue), Qt.blue, brushAlpha=100, size=6, enableLegend=True, xData=list(blue_xdata), yData=list(blue_ydata), autoScale=True)
        
        self.graph.setAxisScale(QwtPlot.xTop, 0.0, 1.0)
         
        self.graph.replot()
        
        
    def replotMA(self):
        if self.data and self.centered:
            Gc, Rc = self.centered
            self.plotMA(Gc, Rc, self.z_scores, self.zCutoff)
        
        
    def commitIf(self):
        if self.autoCommit and self.changedFlag:
            self.commit()
        else:
            self.changedFlag = True
            
            
    def commit(self):
        if not self.data:
            return
        
        G, R = self.merged_splits
        Gc, Rc = self.centered
        ind1, ind2 = self.split_ind
        
        gfactor = Gc / G
        
        domain = orange.Domain(self.data.domain.attributes, self.data.domain.classVar)
        domain.addmetas(self.data.domain.getmetas())
        
        _, _, axis = self.getSelectedGroup()
        if self.appendZScore and axis == 1:
            attr = orange.FloatVariable("Z-Score")
            if not hasattr(self, "z_score_mid"):
                self.z_score_mid = orange.newmetaid()
            mid = self.z_score_mid
            domain.addmeta(mid, attr)
            
        if self.appendRIValues and axis == 1:
            r_attr = orange.FloatVariable("Log Ratio")
            i_attr = orange.FloatVariable("Intensity")
            if not hasattr(self, "_r_mid"):
                self._r_mid = orange.newmetaid()
                self._i_mid = orange.newmetaid()
            r_mid, i_mid = self._r_mid, self._i_mid
            domain.addmeta(r_mid, r_attr)
            domain.addmeta(i_mid, i_attr)
            
        data = orange.ExampleTable(domain, self.data)
        
        def finite_nonmasked(g):
            return numpy.isfinite(g) and g is not numpy.ma.masked
        
        if axis == 0:
            for i, gf in zip(ind1, gfactor):
                for attr in domain.attributes:
                    if not data[i][attr].isSpecial() and finite_nonmasked(gf):
                        data[i][attr] = data[i][attr] * gf
        else:
            ratio, intensity = obiExpression.ratio_intensity(*self.centered) #*self.merged_splits)
            for ex, gf, r, inten, z in zip(data, gfactor, ratio, intensity, self.z_scores):
                for i in ind1:
                    if not ex[i].isSpecial() and finite_nonmasked(gf):
                        ex[i] = float(ex[i]) * gf
                if self.appendZScore:
                    ex[attr] = z if finite_nonmasked(z) else "?"
                    
                if self.appendRIValues:
                    ex[r_attr] = r if finite_nonmasked(r) else "?"
                    ex[i_attr] = inten if finite_nonmasked(inten) else "?"
        
        filtered_ind = list(numpy.ma.abs(self.z_scores.filled(0)) >= self.zCutoff)
        if axis == 0:
            attrs = [attr for attr, filtered in zip(domain.attributes, filtered_ind) if filtered]
            filtered_domain = orange.Domain(attrs, domain.classVar)
            filtered_domain.addmetas(domain.getmetas())
            filtered_data = orange.ExampleTable(filtered_domain, data)
        else:
            filtered_data = data.select([int(b) for b in filtered_ind])
            
        self.send("Normalized expression array", data)
        self.send("Filtered expression array", filtered_data)
        
        
    def saveGraph(self):
        from Orange.OrangeWidgets.OWDlgs import OWChooseImageSizeDlg
        dlg = OWChooseImageSizeDlg(self.graph, parent=self)
        dlg.exec_()
        
        
if __name__ == "__main__":
    app = QApplication(sys.argv)
    w= OWMAPlot()
    data = orange.ExampleTable(os.path.expanduser("~/GDS1210.tab"))
#    data = orange.ExampleTable(os.path.expanduser("../../../doc/datasets/brown-selected.tab"))
    w.setData(data)
    w.show()
    app.exec_()
        
        
